--
-- PostgreSQL database dump
--
-- Dumped from database version 9.5.3
-- Dumped by pg_dump version 9.5.3
SET statement_timeout = 0;
SET lock_timeout = 0;
-- SET client_encoding = 'WIN1251';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET row_security = off;
--
-- Name: plpgsql; Type: EXTENSION; Schema: -; Owner:
--
CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;
--
-- Name: EXTENSION plpgsql; Type: COMMENT; Schema: -; Owner:
--
COMMENT ON EXTENSION plpgsql IS 'PL/pgSQL procedural language';
--
-- Name: tablefunc; Type: EXTENSION; Schema: -; Owner:
--
CREATE EXTENSION IF NOT EXISTS tablefunc WITH SCHEMA public;
--
-- Name: EXTENSION tablefunc; Type: COMMENT; Schema: -; Owner:
--
COMMENT ON EXTENSION tablefunc IS 'functions that manipulate whole tables, including crosstab';
SET search_path = public, pg_catalog;
--
-- Name: enum_task_state; Type: TYPE; Schema: public; Owner: postgres
--
CREATE TYPE enum_task_state AS ENUM (
    'none',
    'started',
    'paused',
    'finished',
    'completed',
    'failed',
    'unknown',
    'semifailed'
);
ALTER TYPE enum_task_state OWNER TO postgres;
--
-- Name: vram_item; Type: TYPE; Schema: public; Owner: postgres
--
CREATE TYPE vram_item AS (
    name            text,
    vram    bigint
);
ALTER TYPE vram_item OWNER TO postgres;
--
-- Name: add_perf_data(integer, double precision, double precision, double precision, double precision, double precision, double precision); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION add_perf_data(_agent_id integer, _processor_time double precision, _memory_free double precision, _disk_time double precision, _disk_free_mb double precision, _network_bytes_sent double precision, _network_bytes_received double precision) RETURNS void
    LANGUAGE plpgsql
    AS $$
DECLARE
	max_len integer;
	freq double precision;
	dist integer;
	new_pos integer;
	prev_pos integer;
	prev_update timestamp with time zone;
	time_interval interval;
	agent_hostname character varying (128);
BEGIN
	SELECT hostname FROM public.agents INTO agent_hostname WHERE agents.id = _agent_id;
	SELECT frequency, history_len FROM public.agent_perf_setup INTO freq, max_len;
	-- Add agent string if it not exists
	INSERT INTO public.agent_perf_data (hostname)
		SELECT agent_hostname WHERE NOT EXISTS (SELECT hostname FROM public.agent_perf_data WHERE hostname = agent_hostname);
	SELECT last_update, last_pos FROM public.agent_perf_data INTO prev_update, prev_pos WHERE hostname = agent_hostname;
	IF prev_pos > 0 THEN
		dist = round(freq * EXTRACT(SECONDS FROM now() - prev_update));
	ELSE
		dist = max_len;
	END IF;
	-- RAISE NOTICE 'add_perf_data(): f: % n: % pos: % t: % s: % d: %', freq, max_len, prev_pos, prev_update, EXTRACT(SECONDS FROM now() - prev_update), dist;
	IF dist >= max_len THEN
		new_pos = 1;
	ELSE
		new_pos = (prev_pos + dist - 1) % max_len + 1;
		IF new_pos < prev_pos THEN -- New cycle
			PERFORM clear_perf_data(_agent_id, prev_pos + 1, max_len);
			PERFORM clear_perf_data(_agent_id, 1, new_pos - 1);
		ELSE
			PERFORM clear_perf_data(_agent_id, prev_pos + 1, new_pos - 1);
		END IF;
	END IF;
	UPDATE public.agent_perf_data SET
		last_pos = new_pos,
		last_update = now(),
		processor_time[new_pos] = _processor_time,
		memory_free[new_pos] = _memory_free,
		disk_time[new_pos] = _disk_time,
		disk_free_mb[new_pos] = _disk_free_mb,
		network_bytes_sent[new_pos] = _network_bytes_sent,
		network_bytes_received[new_pos] = _network_bytes_received
		WHERE hostname = agent_hostname;
	RETURN;
END;$$;
ALTER FUNCTION public.add_perf_data(_agent_id integer, _processor_time double precision, _memory_free double precision, _disk_time double precision, _disk_free_mb double precision, _network_bytes_sent double precision, _network_bytes_received double precision) OWNER TO postgres;
--
-- Name: after_log_insert(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION after_log_insert() RETURNS trigger
    LANGUAGE plpgsql
    AS $$BEGIN
IF (new.msg_type = 'W') THEN
UPDATE sessions SET has_warnings = TRUE
WHERE sessions.id_session = new.id_session;
END IF;
return NULL;
END;$$;
ALTER FUNCTION after_log_insert() OWNER TO postgres;
--
-- Name: agent_has_failed_task(integer, integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION agent_has_failed_task(input_id_agent integer, input_id_task integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$
DECLARE agent_failed_tasks_count integer;
BEGIN
SELECT COUNT(*)
    INTO agent_failed_tasks_count
	FROM failed_tasks
	WHERE failed_tasks.id_task = input_id_task
	AND failed_tasks.id_agent = input_id_agent;

RETURN agent_failed_tasks_count <> 0;

END;
$$;
ALTER FUNCTION public.agent_has_failed_task(input_id_agent integer, input_id_task integer) OWNER TO postgres;
--
-- Name: agent_has_launched_mt_tasks(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION agent_has_launched_mt_tasks(intput_id_agent integer) RETURNS boolean
    LANGUAGE plpgsql
    AS $$DECLARE multi_thread_tasks INTEGER;
BEGIN
	SELECT count(*) INTO multi_thread_tasks
          FROM agents_alive_task_manager
         WHERE agents_alive_task_manager.id_agent = intput_id_agent
		AND multi_threading <> 0
		AND (status = get_task_status('none'::character varying)
		OR status = get_task_status('started'::character varying)
		OR status = get_task_status('paused'::character varying)
		OR status = get_task_status('unknown'::character varying));
	RETURN multi_thread_tasks <> 0;
END;$$;
ALTER FUNCTION public.agent_has_launched_mt_tasks(intput_id_agent integer) OWNER TO postgres;
--
-- Name: agent_perf_setup_changed(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION agent_perf_setup_changed() RETURNS trigger
    LANGUAGE plpgsql
    AS $$BEGIN
	-- RAISE NOTICE 'agent_perf_setup_changed()';
	UPDATE public.agent_perf_data SET
		last_pos = 0,
		processor_time = NULL,
		memory_free = NULL,
		disk_time = NULL,
		disk_free_mb = NULL,
		network_bytes_sent = NULL,
		network_bytes_received = NULL;
	perform pg_notify('agent_perf_setup_changed', jsonb_build_object('frequency', NEW.frequency)::text);
	RETURN NULL;
END;$$;
ALTER FUNCTION public.agent_perf_setup_changed() OWNER TO postgres;
--
-- Name: agregate_progress_and_status(character varying); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION agregate_progress_and_status(in_block_description character varying, OUT out_progress integer, OUT out_status integer) RETURNS record
    LANGUAGE plpgsql IMMUTABLE LEAKPROOF
    AS $$DECLARE ret record;
        prog INT;

        totalrecs int;
        totalfinished int;

        stat INT;
        totalnone int;
BEGIN
  out_progress=0;
  totalrecs=0;
  totalfinished=0;
  totalnone=0;

  for prog, stat in select monitor_processing.progress, monitor_processing.status from monitor_processing where monitor_processing.block_description=in_block_description loop
    if(prog is not null) then
	out_progress=out_progress+prog;
    end if;
    totalrecs = totalrecs+1;
    if(stat=4) then
	out_status=4;
    end if;
    if(out_status is null) then
	if(stat=2 or stat=6 or stat=8 or stat=5) then
	    out_status=stat;
	else
	    if(stat=3) then
	       totalfinished=totalfinished+1;
	    else
		if(stat=0) then
		    totalnone=totalnone+1;
		end if;
	    end if;
	end if;
    end if;
  end loop;
  if(out_status is null) then
    if(totalfinished=totalrecs) then
	out_status=3;
	out_progress = 100;
	return;
    else
	if(totalnone<totalrecs) then
	    out_status=1;
	else
	    out_status=0;
	end if;
    end if;
  end if;
  if(totalrecs>0) then
    out_progress=out_progress/totalrecs;
  else
    out_progress=0;
    out_status=0;
  end if;
  return;
END$$;
ALTER FUNCTION public.agregate_progress_and_status(in_block_description character varying, OUT out_progress integer, OUT out_status integer) OWNER TO postgres;
--
-- Name: apc_refresh_materialized_view(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION apc_refresh_materialized_view() RETURNS trigger
    LANGUAGE plpgsql
    AS $$DECLARE max_time timestamp with time zone;
		populated bool;
BEGIN
	SELECT relispopulated FROM pg_class WHERE relname = 'avg_group_performance_measurements' INTO populated;
	IF populated THEN
		SELECT max(avg_measurement_time) INTO max_time FROM public.avg_group_performance_measurements;
		IF now() - max_time > '00:00:10'::interval THEN
			REFRESH MATERIALIZED VIEW CONCURRENTLY public.avg_group_performance_measurements;
			--REFRESH MATERIALIZED VIEW public.log_performance;
		END IF;
	ELSE
		REFRESH MATERIALIZED VIEW  public.avg_group_performance_measurements;
	END IF;
	RETURN NULL;
END;$$;
ALTER FUNCTION public.apc_refresh_materialized_view() OWNER TO postgres;
--
-- Name: before_agents_insert(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION before_agents_insert() RETURNS trigger
    LANGUAGE plpgsql
    AS $$DECLARE rec RECORD;
BEGIN
 SELECT * FROM agents into rec
 WHERE NEW.hostname = agents.hostname
 ORDER BY agents.id DESC LIMIT 1;
 IF FOUND THEN
	IF (rec.fake_cpu_count < NEW.max_cpu_count) THEN
    	NEW.fake_cpu_count:= rec.fake_cpu_count;
	ELSE
		NEW.fake_cpu_count:=NEW.max_cpu_count;
	END IF;
	IF (rec.mt_coeff < NEW.fake_cpu_count) THEN
		NEW.mt_coeff := rec.mt_coeff;
	ELSE
		NEW.mt_coeff := NEW.fake_cpu_count;
	END IF;
	IF ((rec.gpu_memory).vram > 0 AND (NEW.gpu_memory).vram > 0 AND (rec.gpu_memory).name <> (NEW.gpu_memory).name) THEN
		NEW.gpu_memory= rec.gpu_memory;
		perform pg_notify('table_agents_gpu_changed', jsonb_build_object('gpu_memory', NEW.gpu_memory)::text);
	END IF;
	NEW.is_paused = rec.is_paused;
 ELSE
	NEW.mt_coeff = greatest(least(floor(NEW.cpu_count/4::double precision),floor(NEW.ram_size/(8*1024*1024*1024::double precision))), 1);
	IF ((NEW.gpu_memory).vram > 0) THEN
	NEW.mt_coeff = greatest(least(NEW.mt_coeff, floor((NEW.gpu_memory).vram/(1*1024*1024*1024::double precision))), 1 );
	END IF;
	IF NEW.fake_cpu_count IS NULL THEN
		IF (NEW.cpu_count <= NEW.max_cpu_count) THEN
			NEW.fake_cpu_count:= NEW.cpu_count;
		ELSE
			NEW.fake_cpu_count:=NEW.max_cpu_count;
		END IF;
	END IF;
 END IF;
  RETURN NEW;
END;
$$;
ALTER FUNCTION public.before_agents_insert() OWNER TO postgres;
--
-- Name: before_processing_update(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION before_processing_update() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
BEGIN
	new.update_time = now();
	RETURN new;
END;$$;
ALTER FUNCTION public.before_processing_update() OWNER TO postgres;
--
-- Name: before_status_update(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION before_status_update() RETURNS trigger
    LANGUAGE plpgsql
    AS $$DECLARE task_id integer;
BEGIN
	IF (old.status = new.status) THEN
		RETURN NULL;
	END IF;
	IF (new.status = get_task_status('failed')) THEN
			SELECT id_task INTO task_id FROM public.sessions WHERE id_session = new.id_session;
			new.status = check_failed_tasks(task_id);
	END IF;
	RETURN new;
END;
$$;
ALTER FUNCTION public.before_status_update() OWNER TO postgres;
--
-- Name: calc_current_phase_progress(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION calc_current_phase_progress() RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE result Integer;
l_exst Integer;
all_tasks_count Integer;
progress_sum integer;
BEGIN
	SELECT COUNT(*)
    INTO l_exst
	FROM current_phase_tasks cpt
	WHERE (cpt.status = 4 OR cpt.status = 5 OR cpt.status = 8)
	LIMIT 1;

	IF l_exst = 0 THEN

	SELECT SUM(progress)
	INTO progress_sum
	FROM current_phase_tasks;

	SELECT COUNT(*)
	INTO all_tasks_count
	FROM current_phase_tasks;

	result = (progress_sum::float/NULLIF(all_tasks_count, 0))::integer;
		IF result IS NOT NULL THEN
		result = LEAST(result, 100);
		END IF;
	ELSE
    result = null;
	END IF;

	RETURN result;
END;$$;
ALTER FUNCTION public.calc_current_phase_progress() OWNER TO postgres;
--
-- Name: FUNCTION calc_current_phase_progress(); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION calc_current_phase_progress() IS 'Calculating progress for the current stage';
--
-- Name: calc_host_total_progress(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION calc_host_total_progress(input_hostpid integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$
DECLARE result Integer;
l_exst Integer;
all_tasks_count Integer;
processed_tasks_count Integer;
progress_sum integer;
BEGIN
	SELECT COUNT(*)
    INTO l_exst
	FROM monitor_processing mp
	WHERE (mp.status = 4 OR mp.status = 5) AND mp.hostpid = input_hostpid
	LIMIT 1;

	IF l_exst = 0 THEN

		SELECT SUM(progress)
		INTO progress_sum
		FROM monitor_processing
		WHERE monitor_processing.hostpid = input_hostpid;

		SELECT COUNT(*)
		INTO all_tasks_count
		FROM monitor_processing
		WHERE monitor_processing.hostpid = input_hostpid;

		SELECT COUNT(*)
		INTO processed_tasks_count
		FROM monitor_processing
		WHERE status = 3
		AND monitor_processing.hostpid = input_hostpid;

		IF processed_tasks_count = all_tasks_count THEN
			result = 100;
		ELSE
			result = (progress_sum::float/NULLIF(all_tasks_count, 0))::integer;
			IF result IS NOT NULL THEN
				result = LEAST(result, 100);
			END IF;
		END IF;
	ELSE
		result = null;
	END IF;

	RETURN result;
END;
$$;
ALTER FUNCTION public.calc_host_total_progress(input_hostpid integer) OWNER TO postgres;
--
-- Name: calc_total_progress(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION calc_total_progress() RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE result Integer;
l_exst Integer;
all_tasks_count Integer;
processed_tasks_count Integer;
progress_sum integer;
BEGIN
	SELECT COUNT(*)
    INTO l_exst
	FROM monitor_processing mp
	WHERE (mp.status = 4 OR mp.status = 5)
	LIMIT 1;

	IF l_exst = 0 THEN

		SELECT SUM(progress)
		INTO progress_sum
		FROM monitor_processing;

		SELECT COUNT(*)
		INTO all_tasks_count
		FROM monitor_processing;

		SELECT COUNT(*)
		INTO processed_tasks_count
		FROM monitor_processing
		WHERE status = 3;

		IF processed_tasks_count = all_tasks_count THEN
			result = 100;
		ELSE
			result = (progress_sum::float/NULLIF(all_tasks_count, 0))::integer;
			IF result IS NOT NULL THEN
				result = LEAST(result, 100);
			END IF;
		END IF;
	ELSE
		result = null;
	END IF;

	RETURN result;
END;$$;
ALTER FUNCTION public.calc_total_progress() OWNER TO postgres;
--
-- Name: FUNCTION calc_total_progress(); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION calc_total_progress() IS 'Total Progress Calculation';
--
-- Name: calc_total_progress_and_time_left(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION calc_total_progress_and_time_left() RETURNS json
    LANGUAGE plpgsql
    AS $$DECLARE result json;
l_exst Integer;
all_tasks_count Integer;
progress_sum integer;
start_time timestamp;
total_progress integer;
finished_tasks_count Integer;
time_result interval;
BEGIN
total_progress = calc_total_progress();
SELECT COUNT(*)
    INTO l_exst
	FROM monitor_processing mp
	WHERE (mp.status = 4 OR mp.status = 5 OR mp.status = 2 OR mp.status = 8)
	LIMIT 1;

IF l_exst = 0 THEN

	SELECT s.start_time
	INTO start_time
	FROM monitor_processing s
	WHERE (s.status = 1 OR s.status = 3 OR s.status = 7 OR s.status = 9)
	ORDER BY s.start_time
	LIMIT 1;

	SELECT SUM(progress)
	INTO progress_sum
	FROM monitor_processing;

	SELECT COUNT(*)
	INTO all_tasks_count
	FROM monitor_processing;

	SELECT COUNT(*)
	INTO finished_tasks_count
	FROM monitor_processing mp
	WHERE mp.status = 3;


	time_result = (all_tasks_count*100 - progress_sum)*(clock_timestamp() - start_time)/NULLIF(progress_sum, 0);
	IF(all_tasks_count = finished_tasks_count) THEN
		time_result = null;
	END IF;

	ELSE
    time_result = null;
	END IF;
result := jsonb_build_object('total progress', total_progress, 'time left', time_result);
return result;
END;$$;
ALTER FUNCTION public.calc_total_progress_and_time_left() OWNER TO postgres;
--
-- Name: calc_used_ram_size(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION calc_used_ram_size(input_agent_id integer) RETURNS bigint
    LANGUAGE plpgsql
    AS $$DECLARE used_ram_size bigint;
BEGIN
	SELECT sum(agents_alive_task_manager.ram_per_cpu_needed) INTO used_ram_size
	FROM agents_alive_task_manager
	WHERE agents_alive_task_manager.id_agent = input_agent_id
		AND (status = get_task_status('none'::character varying)
			OR status = get_task_status('started'::character varying)
			OR status = get_task_status('paused'::character varying)
			OR status = get_task_status('unknown'::character varying));
	IF used_ram_size IS NULL THEN
		used_ram_size = 0;
	END IF;
	RETURN used_ram_size;
END;$$;
ALTER FUNCTION public.calc_used_ram_size(input_agent_id integer) OWNER TO postgres;
--
-- Name: check_failed_tasks(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION check_failed_tasks(input_id_task integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE failed_tasks_count integer;
rec RECORD;
agents_alive_count integer;
BEGIN
SELECT COUNT(*)
    INTO failed_tasks_count
	FROM failed_tasks
	WHERE failed_tasks.id_task = input_id_task;

SELECT COUNT(*)
    INTO agents_alive_count
	FROM agents_alive WHERE status = FALSE;

    IF failed_tasks_count = 0 OR failed_tasks_count < agents_alive_count THEN
		SELECT id_session, id_agent INTO rec
		FROM sessions
		WHERE id_task = input_id_task
		ORDER BY id_session DESC LIMIT 1;

		INSERT INTO failed_tasks (id_task, id_agent)
		VALUES (input_id_task, rec.id_agent);

		 IF failed_tasks_count + 1 < agents_alive_count THEN
			RETURN get_task_status('restarted');
	     ELSE
		  RETURN get_task_status('failed');
		 END IF;
	END IF;
END;$$;
ALTER FUNCTION public.check_failed_tasks(input_id_task integer) OWNER TO postgres;
--
-- Name: check_tasks_fidelity(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION check_tasks_fidelity(agent_id integer) RETURNS void
    LANGUAGE plpgsql
    AS $$DECLARE task_info RECORD;
task_locked bool;
BEGIN
FOR task_info in
(SELECT * FROM monitor_processing
WHERE monitor_processing.status = 1 OR monitor_processing.status = 2)
LOOP
	IF (task_info.id_agent <> agent_id) THEN
		task_locked=pg_try_advisory_lock(1, task_info.id_task);

		IF (task_locked) THEN

		perform pg_advisory_unlock(1, task_info.id_task);


		UPDATE processing
		SET status=get_task_status('restarted')
		WHERE processing.id_session = task_info.id_session;


		UPDATE agents

		SET is_alive = false

		WHERE agents.id = task_info.id_agent;

		END IF;

	END IF;


END LOOP;
END;$$;
ALTER FUNCTION public.check_tasks_fidelity(agent_id integer) OWNER TO postgres;
--
-- Name: clear_perf_data(integer, integer, integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION clear_perf_data(_agent_id integer, lower_bound integer, upper_bound integer) RETURNS void
    LANGUAGE plpgsql
    AS $$
DECLARE
	null_array double precision[];
	agent_hostname character varying (128);
BEGIN
	--RAISE NOTICE 'clear_perf_data(): lb: % ub: %', lower_bound, upper_bound;
	IF lower_bound <= upper_bound THEN
		SELECT hostname FROM public.agents INTO agent_hostname WHERE agents.id = _agent_id;
		null_array = array_fill(NULL::double precision, ARRAY[upper_bound - lower_bound + 1])::double precision[];
		UPDATE public.agent_perf_data SET
			processor_time[lower_bound:upper_bound] = null_array,
			memory_free[lower_bound:upper_bound] = null_array,
			disk_time[lower_bound:upper_bound] = null_array,
			disk_free_mb[lower_bound:upper_bound] = null_array,
			network_bytes_sent[lower_bound:upper_bound] = null_array,
			network_bytes_received[lower_bound:upper_bound] = null_array
			WHERE hostname = agent_hostname;
	END IF;
	RETURN;
END;$$;
ALTER FUNCTION public.clear_perf_data(_agent_id integer, lower_bound integer, upper_bound integer) OWNER TO postgres;
--
-- Name: FUNCTION clear_perf_data(_agent_id integer, lower_bound integer, upper_bound integer); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION clear_perf_data(_agent_id integer, lower_bound integer, upper_bound integer) IS 'Helper function. Assigns NULL to the performance counters in the passed interval.';
--
-- Name: count_free_agents(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION count_free_agents() RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE free_agents INTEGER;
BEGIN
SELECT count(*) INTO free_agents
  FROM agents_alive_task_count
 WHERE task_count < cpu_count;
RETURN free_agents;
END;$$;
ALTER FUNCTION public.count_free_agents() OWNER TO postgres;
--
-- Name: count_launched_tasks(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION count_launched_tasks(input_id_agent integer, exclude_observers boolean DEFAULT false) RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE task_count INTEGER;
BEGIN
IF exclude_observers THEN
SELECT count(*) INTO task_count
	FROM agents_alive_task_manager
	WHERE agents_alive_task_manager.id_agent = input_id_agent
	AND agents_alive_task_manager.multi_threading <> 2
	AND (status = get_task_status('none'::character varying)
	OR status = get_task_status('started'::character varying)
	OR status = get_task_status('paused'::character varying)
	OR status = get_task_status('unknown'::character varying));
ELSE
	SELECT count(*) INTO task_count
	FROM agents_alive_task_manager
	WHERE agents_alive_task_manager.id_agent = input_id_agent
	AND (status = get_task_status('none'::character varying)
	OR status = get_task_status('started'::character varying)
	OR status = get_task_status('paused'::character varying)
	OR status = get_task_status('unknown'::character varying));
END IF;
RETURN task_count;
END;$$;
ALTER FUNCTION public.count_launched_tasks(input_id_agent integer, exclude_observers boolean) OWNER TO postgres;
--
-- Name: dbg_perf_data(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION dbg_perf_data(_agent_id integer) RETURNS void
    LANGUAGE plpgsql
    AS $$
DECLARE
	dbg_array double precision[];
	dbg_array2 double precision[];
	dbg_array3 double precision[];
	dbg_array4 double precision[];
	dbg_array5 double precision[];
	dbg_array6 double precision[];
	agent_hostname character varying (128);
	lower_bound integer;
	upper_bound integer;
	cnt integer;
BEGIN
	lower_bound = 1;
	upper_bound = 604800;
	--upper_bound = 100000;
	cnt = 1;
	LOOP
		EXIT WHEN cnt > upper_bound;
		dbg_array[cnt] = cnt;
		dbg_array2[cnt] = cnt*2;
		dbg_array3[cnt] = cnt*3;
		dbg_array4[cnt] = cnt*4;
		dbg_array5[cnt] = cnt*5;
		dbg_array6[cnt] = cnt*6;
		cnt = cnt + 1;
	END LOOP;
	SELECT hostname FROM public.agents INTO agent_hostname WHERE agents.id = _agent_id;
	-- dbg_array = array_fill(12.345::double precision, ARRAY[upper_bound - lower_bound + 1])::double precision[];
	UPDATE public.agent_perf_data SET
		processor_time = dbg_array,
		memory_free = dbg_array2,
		disk_time = dbg_array3,
		disk_free_mb = dbg_array4,
		network_bytes_sent = dbg_array5,
		network_bytes_received = dbg_array6
		WHERE hostname = agent_hostname;
END;$$;
ALTER FUNCTION public.dbg_perf_data(_agent_id integer) OWNER TO postgres;
CREATE FUNCTION public.delete_tasks(delete_only_finished boolean) RETURNS void
    LANGUAGE plpgsql
    AS $$BEGIN
IF NOT delete_only_finished THEN
insert into public.user_monitor_commands (id_task, command_type)
select id_task, get_command_type('abort_delete') AS command_type
from monitor_processing
where status = get_task_status('paused')
OR status = get_task_status('started')
OR status = get_task_status('resumed');

insert into public.user_monitor_commands (id_task, command_type)
select id_task, get_command_type('delete') AS command_type
from monitor_processing
where status <> get_task_status('paused')
AND status <> get_task_status('started')
AND status <> get_task_status('resumed')
OR status IS NULL;
ELSE
insert into public.user_monitor_commands (id_task, command_type)
select id_task, get_command_type('delete') AS command_type
from monitor_processing
where status = get_task_status('finished')
OR status = get_task_status('failed')
OR status = get_task_status('stopped');
END IF;
END$$;


ALTER FUNCTION public.delete_tasks(delete_only_finished boolean) OWNER TO postgres;
--
-- Name: evaluate_current_phase_time(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION evaluate_current_phase_time() RETURNS interval
    LANGUAGE plpgsql
    AS $$DECLARE result interval;
l_exst Integer;
all_tasks_count Integer;
start_time timestamp;
progress_sum integer;
BEGIN
	SELECT COUNT(*)
    INTO l_exst
	FROM current_phase_tasks cpt
	WHERE (cpt.status = 4 OR cpt.status = 5 OR cpt.status = 2 OR cpt.status = 8)
	LIMIT 1;

	IF l_exst = 0 THEN

	SELECT s.start_time
	INTO start_time
	FROM current_phase_tasks s
	WHERE (s.status = 1 OR s.status = 3 OR s.status = 7 OR s.status = 9)
	ORDER BY s.start_time
	LIMIT 1;

	SELECT SUM(progress)
	INTO progress_sum
	FROM current_phase_tasks;

	SELECT COUNT(*)
	INTO all_tasks_count
	FROM current_phase_tasks;

	result = (all_tasks_count*100 - progress_sum)*(clock_timestamp() - start_time)/NULLIF(progress_sum, 0);
	ELSE
    result = null;
	END IF;

	RETURN result;
END;
$$;
ALTER FUNCTION public.evaluate_current_phase_time() OWNER TO postgres;
--
-- Name: FUNCTION evaluate_current_phase_time(); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION evaluate_current_phase_time() IS 'Estimate of the remaining time until the completion of the current stage';
--
-- Name: execute_user_command(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION execute_user_command() RETURNS trigger
    LANGUAGE plpgsql
    AS $$DECLARE session_id integer;
BEGIN
	CASE (new.command_type)
	WHEN get_command_type('pause') THEN
		perform pg_notify('pause_batch_exec', lpad(CAST(NEW.id_task AS TEXT), 16, '0'));
	WHEN get_command_type('start') THEN
		DELETE FROM failed_tasks
		WHERE failed_tasks.id_task = NEW.id_task;
		SELECT id_session INTO session_id
		FROM public.last_sessions
		WHERE id_task = new.id_task;

		IF (session_id IS NULL)  THEN
		RAISE EXCEPTION 'Empty session for task %', new.id_task;
		RETURN NULL;
		END IF;

		UPDATE public.processing
		SET    status = get_task_status('restarted')
		WHERE  id_session = session_id;
	WHEN get_command_type('abort') THEN
		perform pg_notify('stop_batch_exec', lpad(CAST(NEW.id_task AS TEXT), 16, '0'));
	WHEN get_command_type('abort_delete') THEN
		perform pg_notify('stop_delete_batch_exec', lpad(CAST(NEW.id_task AS TEXT), 16, '0'));
	WHEN get_command_type('resume') THEN
		perform pg_notify('resume_batch_exec', lpad(CAST(NEW.id_task AS TEXT), 16, '0'));
	WHEN get_command_type('delete') THEN
		DELETE FROM public.tasks
			WHERE id = new.id_task;
		DELETE FROM public.task_dependencies
			WHERE id_task = new.id_task;
		DELETE FROM failed_tasks
		WHERE failed_tasks.id_task = NEW.id_task;
	ELSE
		RAISE EXCEPTION 'Unknown user command %', new.command_type;
	END CASE;
	RETURN NULL;
END;$$;
ALTER FUNCTION public.execute_user_command() OWNER TO postgres;
--
-- Name: find_agent(bigint, integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION find_agent(ram_per_cpu bigint, multi_threading_required integer DEFAULT 0) RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE agent_id INTEGER;
BEGIN
IF multi_threading_required = 0 THEN
	SELECT id INTO agent_id FROM public.agents_alive_task_count
	WHERE (is_paused IS NOT TRUE AND ram_size >= ram_per_cpu * cpu_count
	AND task_count < cpu_count) ORDER BY task_count ASC LIMIT 1;
ELSE
	SELECT id INTO agent_id FROM public.agents_alive_task_count
	WHERE (is_paused IS NOT TRUE AND task_count < cpu_count)
	ORDER BY (cpu_count - task_count) DESC LIMIT 1;
END IF;
RETURN agent_id;
END;$$;
ALTER FUNCTION public.find_agent(ram_per_cpu bigint, multi_threading_required integer) OWNER TO postgres;
--
-- Name: find_appropriate_agent(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION find_appropriate_agent() RETURNS trigger
    LANGUAGE plpgsql
    AS $$DECLARE
  agent_id  int;
  finished_parents int;
  total_parents int;
  session_id int;
BEGIN
	agent_id = public.find_agent(new.ram_per_cpu_needed, new.multi_threading);

	IF (agent_id IS NULL) THEN
		RAISE WARNING 'NO APPROPRIATE AGENT WAS FOUND for task %', new.id;
		RETURN NULL;
	END IF;
	SELECT count(*) INTO finished_parents
	FROM public.task_parents_status
	WHERE (id_task = new.id)
		AND (task_status IS NULL)
		AND (parent_status = get_task_status('finished'));
	SELECT count(*) INTO total_parents
	FROM public.task_parents
	WHERE (id_task = new.id);

	IF (finished_parents = total_parents) THEN
		RAISE NOTICE 'find_appropriate_agent: Inserting task %', new.id;

		INSERT INTO public.sessions (id_task, id_agent)
		VALUES (new.id, agent_id) RETURNING id_session INTO STRICT session_id;
		INSERT INTO public.processing (id_session, progress, status)
		VALUES (session_id, 0, get_task_status('started'));

	ELSE
		RAISE NOTICE 'find_appropriate_agent: id_task=%, finished_parents=%, total_parents=%',new.id, finished_parents, total_parents;
	END IF;
	RETURN NULL;
END;$$;
ALTER FUNCTION public.find_appropriate_agent() OWNER TO postgres;
--
-- Name: FUNCTION find_appropriate_agent(); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION find_appropriate_agent() IS 'Find a suitable agent according to the characteristics for the execution of the current task. Fires when a row is inserted into the tasks table.';
--
-- Name: find_apropriate_tasks(bigint, integer, integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION find_apropriate_tasks(ram_per_cpu bigint, agent_id integer, multi_threading_required integer DEFAULT 0) RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE o_agent_id INTEGER;
BEGIN
IF multi_threading_required = 0 THEN
	SELECT id INTO o_agent_id FROM public.agents_alive_task_count
	WHERE (is_paused IS NOT TRUE AND ram_size >= ram_per_cpu * cpu_count
	AND count_launched_tasks(id, true) < cpu_count
	and id = agent_id)
	ORDER BY task_count ASC LIMIT 1;
ELSEIF multi_threading_required = 2 THEN
	SELECT id INTO o_agent_id FROM public.agents_alive_task_count
	WHERE (is_paused IS NOT TRUE AND count_launched_tasks(id) < cpu_count and id = agent_id)
	ORDER BY (cpu_count - task_count) DESC LIMIT 1;
ELSE
	SELECT id INTO o_agent_id FROM public.agents_alive_task_count
	WHERE (is_paused IS NOT TRUE AND count_launched_tasks(id, true) < mt_coeff and id = agent_id)
	ORDER BY (cpu_count - task_count) DESC LIMIT 1;
END IF;
RETURN o_agent_id;
END;$$;
ALTER FUNCTION public.find_apropriate_tasks(ram_per_cpu bigint, agent_id integer, multi_threading_required integer) OWNER TO postgres;
--
-- Name: find_task_for_new_agent(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION find_task_for_new_agent() RETURNS trigger
    LANGUAGE plpgsql
    AS $$DECLARE task_ids_iterator
		CURSOR FOR SELECT DISTINCT ON (id_task) *
			   FROM task_status
			   WHERE (status IS NULL
			       OR status = get_task_status('restarted'));
			   --ORDER BY status DESC;
	task_info RECORD;
	agent_id INT;
	total_parents INT;
	finished_parents INT;
	session_id INT;
BEGIN
-- Main scheduler loop. Go through all the tasks that are not running and need to be restarted.
-- Find a suitable agent and check task dependencies.
-- If the agent is found and all parents are completed or missing, then start/restart the task.
	FOR task_info IN task_ids_iterator LOOP
		IF (public.count_free_agents() = 0) THEN
			RAISE NOTICE 'There are no free agents!';
			RETURN NULL;
		END IF;
		agent_id = public.find_agent(task_info.ram_per_cpu_needed, task_info.multi_threading);
		IF (agent_id IS NULL) THEN
			RAISE NOTICE 'On task % finished: No appropriate agent was found for task %', new.id_session, task_info.id_task;
			CONTINUE;
		END IF;
		SELECT count(*) INTO finished_parents
		FROM public.task_parents_status
		WHERE (id_task = task_info.id_task)
			--AND (task_status IS NULL)
			AND (parent_status = get_task_status('finished'));
		SELECT count(*) INTO total_parents
		FROM public.task_parents
		WHERE (id_task = task_info.id_task);

		IF (finished_parents <> total_parents) THEN
			RAISE NOTICE 'id_task=%, finished_parents=%, total_parents=%',task_info.id_task, finished_parents, total_parents;
			CONTINUE;
		END IF;
		IF (task_info.status IS NULL OR task_info.status = get_task_status('restarted')) THEN

			INSERT INTO public.sessions (id_task, id_agent)
			VALUES (task_info.id_task, agent_id) RETURNING id_session INTO STRICT session_id;
			INSERT INTO public.processing (id_session, progress, status)
			VALUES (session_id, 0, get_task_status('started'));
		END IF;

	END LOOP;
	RETURN NULL;
END;$$;
ALTER FUNCTION public.find_task_for_new_agent() OWNER TO postgres;
--
-- Name: get_command_type(character varying); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION get_command_type(cmd_name character varying) RETURNS integer
    LANGUAGE plpgsql IMMUTABLE
    AS $$declare res_id integer;
begin
	select id_command into res_id from public.command_type_description
	where command_name = cmd_name;
	return res_id;
end;$$;
ALTER FUNCTION public.get_command_type(cmd_name character varying) OWNER TO postgres;
--
-- Name: FUNCTION get_command_type(cmd_name character varying); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION get_command_type(cmd_name character varying) IS 'Get user command code
';
--
-- Name: get_res_id(character varying); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION public.get_res_id(i_res_name character varying) RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE o_id INTEGER;
BEGIN
	 INSERT INTO public.resource_identificators (res_name, status) VALUES (i_res_name, 'inserted')
	 ON CONFLICT ON CONSTRAINT pkey_resource_identificators
	 DO UPDATE SET status = 'upserted' RETURNING id INTO o_id;
	RETURN o_id;
END;$$;
ALTER FUNCTION public.get_res_id(i_res_name character varying) OWNER TO postgres;
--
-- Name: get_task_status(character varying); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION get_task_status(st_name character varying) RETURNS integer
    LANGUAGE plpgsql IMMUTABLE
    AS $$declare status_id integer;
begin
	select id into status_id from public.task_status_description
	where status_name = st_name;
	return status_id;
end;$$;
ALTER FUNCTION public.get_task_status(st_name character varying) OWNER TO postgres;
--
-- Name: FUNCTION get_task_status(st_name character varying); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION get_task_status(st_name character varying) IS 'Get task status code by name';
--
-- Name: get_total_cpu_count; Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION public.get_total_cpu_count() RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE cpu_count_total integer;
BEGIN
SELECT INTO cpu_count_total COALESCE(SUM(cpu_count), 0) FROM agents_alive;
return cpu_count_total;
END;$$;
ALTER FUNCTION public.get_total_cpu_count() OWNER TO postgres;
--
-- Name: is_manual_mode(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION is_manual_mode() RETURNS boolean
    LANGUAGE plpgsql
    AS $$BEGIN
 return manual_mode
   FROM processing_parameters LIMIT 1;
END;
$$;
ALTER FUNCTION public.is_manual_mode() OWNER TO postgres;
--
-- Name: lock_resource(character varying, boolean); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION lock_resource(file_name character varying, need_exclusive boolean) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE
    AS $$declare result boolean default FALSE;
	      file_uid int;
begin
	file_uid = public.get_res_id(file_name);
	IF (file_uid IS NOT NULL OR file_uid > 0) THEN
		IF (need_exclusive) THEN
			result = pg_try_advisory_lock(0, file_uid);
		ELSE
			result = pg_try_advisory_lock_shared(0, file_uid);
		END IF;

	END IF;
	RETURN result;
end;$$;
ALTER FUNCTION public.lock_resource(file_name character varying, need_exclusive boolean) OWNER TO postgres;
--
-- Name: FUNCTION lock_resource(file_name character varying, need_exclusive boolean); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION lock_resource(file_name character varying, need_exclusive boolean) IS 'Get task status code by name';
--
-- Name: log_performance_refresh_materialized_view(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION log_performance_refresh_materialized_view() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
BEGIN
	REFRESH MATERIALIZED VIEW public.log_performance;
	RETURN NULL;
END;$$;
ALTER FUNCTION public.log_performance_refresh_materialized_view() OWNER TO postgres;
--
-- Name: run_task_planning(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION run_task_planning() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
DECLARE
	task_id INT;
BEGIN
-- In case the task has changed its status to started or none, do nothing,
-- if the task is paused, send the appropriate notification to the agent
-- if the task is stopped, send the appropriate notification to the agent
-- otherwise, run the main scheduler loop.
	IF (new.status IS NULL) THEN
		RAISE WARNING 'Updated session status is null!';
		RETURN NULL;
	END IF;
	CASE (new.status)
		WHEN (get_task_status('none')) THEN
			RETURN NULL;
		WHEN (get_task_status('started')) THEN
			RETURN NULL;
		--WHEN (get_task_status('paused')) THEN
			--perform send_pause_batch_exec(new.id_session);
		--WHEN (get_task_status('stopped')) THEN
			--perform send_stop_batch_exec(new.id_session);
		--WHEN (get_task_status('resumed')) THEN
			--perform send_resume_batch_exec(new.id_session);
		WHEN (get_task_status('failed')) THEN
			--SELECT id_task INTO task_id FROM public.sessions WHERE id_session = new.id_session;
			--perform check_failed_tasks(task_id);
			perform pg_notify('job_finished', ''); -- Let the agent take the next task, if there is a suitable one.
		--WHEN (get_task_status('unknown')) THEN
			--SELECT id_task INTO task_id FROM public.sessions WHERE id_session = new.id_session;
			--perform check_failed_tasks(task_id);
		WHEN (get_task_status('restarted')) THEN
		--
			SELECT id_task INTO task_id FROM public.sessions WHERE id_session = new.id_session;
			perform pg_advisory_unlock(1, task_id);
			perform pg_notify('job_finished', '');
		ELSE
			perform pg_notify('job_finished', '');
	END CASE;
	RETURN NULL;
END;
$$;
ALTER FUNCTION public.run_task_planning() OWNER TO postgres;
--
-- Name: send_notify_on_agent_removed_from_pause(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_agent_removed_from_pause() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
BEGIN
IF (NEW.is_paused IS NOT TRUE)
THEN
perform pg_notify('agent_removed_from_pause', NEW.id::text);
END IF;
return null;
END;
$$;
ALTER FUNCTION public.send_notify_on_agent_removed_from_pause() OWNER TO postgres;
--
-- Name: send_notify_on_agents_change(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_agents_change() RETURNS trigger
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE notify_params TEXT;
begin
	IF(TG_OP = 'UPDATE' OR TG_OP = 'INSERT') THEN
		notify_params = jsonb_build_object( 'id', NEW.id,
		'operation'	, TG_OP,
		'hostname', NEW.hostname,
		'cpu_count', NEW.fake_cpu_count,
		'real_cpu', NEW.cpu_count,
		'ram_size', NEW.ram_size,
		'last_update_time', NEW.last_update_time,
		'mt_coeff', NEW.mt_coeff,
		'is_paused', NEW.is_paused,
		'ip_address', NEW.ip_address,
		'version', NEW.version,
		'gpu_memory', NEW.gpu_memory,
		'avail_gpu_devices', NEW.avail_gpu_devices,
		'task_count', count_launched_tasks(NEW.id, true))::text;
	END IF;
	perform pg_notify('table_agents_changed', notify_params);
	return null;
end;$$;
ALTER FUNCTION public.send_notify_on_agents_change() OWNER TO postgres;
--
-- Name: send_notify_on_agents_gpu_change(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION public.send_notify_on_agents_gpu_change() RETURNS trigger
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE notify_params TEXT;
begin
		notify_params = jsonb_build_object( 
		'gpu_memory', NEW.gpu_memory)::text;
	perform pg_notify('table_agents_gpu_changed', notify_params);
	return null;
end;$$;
ALTER FUNCTION public.send_notify_on_agents_gpu_change() OWNER TO postgres;
--
-- Name: send_notify_on_agents_perf_changed(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_agents_perf_changed() RETURNS trigger
    LANGUAGE plpgsql
    AS $$DECLARE notify_params TEXT;
	rec RECORD;
begin

	IF(TG_OP = 'UPDATE' OR TG_OP = 'INSERT') THEN

	    SELECT agents_perf.*
		FROM agents_perf
		INTO rec
		WHERE agents_perf.hostname = 'Overall Performance';
		notify_params = jsonb_build_object( 'hostname', NEW.hostname,
		'last_update'	, NEW.last_update,
		'processor_time', NEW.processor_time[NEW.last_pos],

		'memory_free', NEW.memory_free[NEW.last_pos],
		'disk_time', NEW.disk_time[NEW.last_pos],
		'disk_free_mb', NEW.disk_free_mb[NEW.last_pos],
		'network_bytes_sent', NEW.network_bytes_sent[NEW.last_pos],
		'network_bytes_received', NEW.network_bytes_received[NEW.last_pos],

		'processor_time_all', rec.processor_time,

		'memory_free_all', rec.memory_free,
		'disk_time_all', rec.disk_time,
		'disk_free_mb_all', rec.disk_free_mb,
		'network_bytes_sent_all', rec.network_bytes_sent,
		'network_bytes_received_all', rec.network_bytes_received)::text;
	END IF;
	perform pg_notify('table_agents_perf_changed', notify_params);
	return null;
end;$$;
ALTER FUNCTION public.send_notify_on_agents_perf_changed() OWNER TO postgres;
--
-- Name: send_notify_on_manual_start_changed(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_manual_start_changed() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
BEGIN
perform pg_notify('manual_start_changed', NEW.manual_start::text);
return null;
END;
$$;
ALTER FUNCTION public.send_notify_on_manual_start_changed() OWNER TO postgres;

--
-- Name: send_notify_on_max_tasks_increased(); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION public.send_notify_on_max_tasks_increased() RETURNS trigger
    LANGUAGE plpgsql
    AS $$BEGIN
IF (new.fake_cpu_count > old.fake_cpu_count OR new.mt_coeff > old.mt_coeff) THEN
perform pg_notify('job_finished', '');
END IF;
return NULL;
END;
$$;


ALTER FUNCTION public.send_notify_on_max_tasks_increased() OWNER TO postgres;

--
-- Name: send_notify_on_new_session(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_new_session() RETURNS trigger
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE

BEGIN
	perform send_start_batch_exec(new.id_session, new.id_task, new.id_agent);
	RETURN NULL;
END;$$;
ALTER FUNCTION public.send_notify_on_new_session() OWNER TO postgres;
--
-- Name: FUNCTION send_notify_on_new_session(); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION send_notify_on_new_session() IS 'A trigger function that fires on inserting a new task into the processing table.
Sends a notification to the client with the identifier of the task that needs to be launched and its parameters.
The client must subscribe to the ''launch'' notification and parse the parameters passed as a single string.
The first 16 characters are the task identifier (filled in front with zeros), the rest are the tasks xml parameters.';
--
-- Name: send_notify_on_processing_change(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_processing_change() RETURNS trigger
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE notify_params TEXT;
	channel_name TEXT;
	rec RECORD;
	rec_session RECORD;
BEGIN
        IF (TG_OP = 'DELETE') THEN
		rec := OLD;
		channel_name = 'table_processing_delete';
        ELSIF (TG_OP = 'UPDATE') THEN
		rec := NEW;
		channel_name = 'table_processing_update';
        ELSIF (TG_OP = 'INSERT') THEN
		rec := NEW;
		channel_name = 'table_processing_insert';
        END IF;

	SELECT sessions.*,
	tasks.name, tasks.priority,
	tasks.hostname, tasks.hostpid,
	tasks.multi_threading,
	agents.hostname AS agent_hostname,
	tasks.command_args,
	tasks.registration_time
	FROM sessions
	LEFT JOIN tasks ON sessions.id_task = tasks.id
	LEFT JOIN agents ON sessions.id_agent = agents.id
	INTO rec_session
	WHERE sessions.id_session = rec.id_session;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'session % not found', rec.id_session;
	END IF;


	notify_params = jsonb_build_object( 'id_session', rec.id_session,
										'id_task', rec_session.id_task,
										'update_time', rec.update_time,
										'progress', rec.progress,
										'status', rec.status,
										'id_agent', rec_session.id_agent,
										'start_time', rec_session.start_time,
										'elapsed_time', rec_session.elapsed_time,
										'name', rec_session.name,
										'hostname', rec_session.hostname,
										'hostpid', rec_session.hostpid,
									    'priority', rec_session.priority,
										'command_args', rec_session.command_args,
										'registration_time', rec_session.registration_time,
										'agent_hostname', rec_session.agent_hostname,
										'multi_threading', rec_session.multi_threading,
										'has_warnings', rec_session.has_warnings)::text;
	perform pg_notify(channel_name, notify_params);
	RETURN NULL;
END;$$;
ALTER FUNCTION public.send_notify_on_processing_change() OWNER TO postgres;
--
-- Name: send_notify_on_processing_mode_change(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_processing_mode_change() RETURNS trigger
    LANGUAGE plpgsql
    AS $$BEGIN
perform pg_notify('processing_mode_changed', NEW.manual_mode::text);
return null;
END;
$$;
ALTER FUNCTION public.send_notify_on_processing_mode_change() OWNER TO postgres;
--
-- Name: send_notify_on_tasks_change(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_notify_on_tasks_change() RETURNS trigger
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE notify_params TEXT;
	rec RECORD;
BEGIN
        IF (TG_OP = 'DELETE') THEN
			IF (SELECT count(*) < 2 FROM tasks WHERE tasks.hostname = OLD.hostname) THEN
				perform pg_notify('table_hosts_changed', '');
			END IF;
		notify_params = jsonb_build_object('id_task', OLD.id)::text;
        perform pg_notify('table_tasks_delete', notify_params);
        ELSE
		IF (TG_OP = 'INSERT' AND (SELECT count(*) < 2 FROM tasks WHERE tasks.hostname = NEW.hostname)) THEN
				perform pg_notify('table_hosts_changed', '');
				END IF;
		SELECT task_dependencies_info.id_group,
		task_dependencies_info.id_dependency
		FROM task_dependencies_info
		INTO rec
		WHERE task_dependencies_info.id_task = NEW.id;
		notify_params = jsonb_build_object('id_task', NEW.id, 'name', NEW.name,
		'hostname', NEW.hostname,'hostpid', NEW.hostpid,
		'multi_threading', NEW.multi_threading, 'registration_time',
		NEW.registration_time, 'priority', NEW.priority, 'id_group', rec.id_group,
		'id_dependency', rec.id_dependency,
		'command_args', NEW.command_args)::text;
        perform pg_notify('table_tasks_changed', notify_params);
        END IF;
	RETURN NULL;
END;$$;
ALTER FUNCTION public.send_notify_on_tasks_change() OWNER TO postgres;
--
-- Name: send_pause_batch_exec(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_pause_batch_exec(input_id_session integer) RETURNS void
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE task_id INTEGER;
	notify_params TEXT;
BEGIN
SELECT id_task INTO task_id FROM public.sessions WHERE id_session = input_id_session;
IF (task_id IS NULL) THEN
	RAISE EXCEPTION 'send_pause_batch_exec: NO INFO ABOUT SESSION %', input_id_session;
	RETURN;
END IF;
notify_params = lpad(CAST(task_id AS TEXT), 16, '0');
perform pg_notify('pause_batch_exec', notify_params);
END;$$;
ALTER FUNCTION public.send_pause_batch_exec(input_id_session integer) OWNER TO postgres;
--
-- Name: send_resume_batch_exec(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_resume_batch_exec(input_id_session integer) RETURNS void
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE task_id INTEGER;
	notify_params TEXT;
BEGIN
SELECT id_task INTO task_id FROM public.sessions WHERE id_session = input_id_session;
IF (task_id IS NULL) THEN
	RAISE EXCEPTION 'send_resume_batch_exec: NO INFO ABOUT SESSION %', input_id_session;
	RETURN;
END IF;
notify_params = lpad(CAST(task_id AS TEXT), 16, '0');
perform pg_notify('resume_batch_exec', notify_params);
END;$$;
ALTER FUNCTION public.send_resume_batch_exec(input_id_session integer) OWNER TO postgres;
--
-- Name: send_start_batch_exec(integer, integer, integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_start_batch_exec(input_id_session integer, input_id_task integer, input_id_agent integer) RETURNS void
    LANGUAGE plpgsql IMMUTABLE
    AS $$DECLARE task_row tasks%ROWTYPE;
	notify_params TEXT;
BEGIN
SELECT * INTO task_row FROM public.tasks WHERE id = input_id_task;
IF (task_row IS NULL) THEN
	RAISE EXCEPTION 'send_start_batch_exec: NO INFO ABOUT TASK %', input_id_task;
	RETURN;
END IF;
notify_params = lpad(CAST(input_id_session AS TEXT), 16, '0') || '|' ||
		lpad(CAST(input_id_task AS TEXT), 16, '0')    || '|' ||
		lpad(CAST(input_id_agent AS TEXT), 16, '0')   || '|' ||
		     CAST(task_row.command AS TEXT)           || '|' ||
		     CAST(task_row.command_args AS TEXT);
perform pg_notify('start_batch_exec', notify_params);
END;$$;
ALTER FUNCTION public.send_start_batch_exec(input_id_session integer, input_id_task integer, input_id_agent integer) OWNER TO postgres;
--
-- Name: send_stop_batch_exec(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION send_stop_batch_exec(input_id_session integer) RETURNS void
    LANGUAGE plpgsql IMMUTABLE
    AS $$
DECLARE task_id INTEGER;
	notify_params TEXT;
BEGIN
SELECT id_task INTO task_id FROM public.sessions WHERE id_session = input_id_session;
IF (task_id IS NULL) THEN
	RAISE EXCEPTION 'send_stop_batch_exec: NO INFO ABOUT SESSION %', input_id_session;
	RETURN;
END IF;
notify_params = lpad(CAST(task_id AS TEXT), 16, '0');
perform pg_notify('stop_batch_exec', notify_params);
END;$$;
ALTER FUNCTION public.send_stop_batch_exec(input_id_session integer) OWNER TO postgres;
--
-- Name: set_fake_cpu_count(integer, integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION public.set_fake_cpu_count(input_id_agent integer, INOUT value_new integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$DECLARE rec RECORD;
BEGIN
SELECT * INTO rec FROM agents WHERE agents.id = input_id_agent;
IF (value_new > rec.max_cpu_count) THEN
value_new = rec.max_cpu_count;
END IF;

UPDATE agents SET fake_cpu_count = value_new WHERE agents.id = rec.id;
END;$$;


ALTER FUNCTION public.set_fake_cpu_count(input_id_agent integer, INOUT value_new integer) OWNER TO postgres;
--
-- Name: set_mt_coeff(double precision, integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION public.set_mt_coeff(GB_per_MT double precision, Cores_per_MT integer) RETURNS void
    LANGUAGE plpgsql
    AS $$
BEGIN
	UPDATE agents
	SET mt_coeff = greatest(least(floor(cpu_count/(Cores_per_MT::double precision)), floor(ram_size/(GB_per_MT*1024*1024*1024))), 1);
END;
$$;
ALTER FUNCTION public.set_mt_coeff(GB_per_MT double precision, Cores_per_MT integer) OWNER TO postgres;

--
-- Name: statistics_host(integer); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION statistics_host(input_hostpid integer) RETURNS TABLE(field_name text, field_value text)
    LANGUAGE plpgsql
    AS $$BEGIN
RETURN QUERY WITH stat_raw AS (
SELECT 'start_time'::text AS field_name,
            (( SELECT min(monitor_processing.start_time) FROM monitor_processing WHERE monitor_processing.hostpid = input_hostpid))::text AS field_value,
            1 AS sort_col
UNION
         SELECT 'finish_time'::text AS field_name,
            ((SELECT sum(elapsed_time) FROM monitor_processing WHERE monitor_processing.hostpid = input_hostpid))::text AS field_value,
            2 AS sort_col
 UNION
         SELECT 'elapsed_time'::text AS field_name,
            ((SELECT max(GREATEST(sessions.finish_time, monitor_processing.start_time + monitor_processing.elapsed_time)) - min(monitor_processing.start_time) FROM monitor_processing JOIN sessions ON monitor_processing.id_task = sessions.id_task  WHERE monitor_processing.hostpid = input_hostpid))::text AS field_value,
            3 AS sort_col
 UNION
         SELECT 'tasks_total'::text AS field_name,
            ((SELECT count(*) FROM monitor_processing WHERE monitor_processing.hostpid = input_hostpid))::text AS field_value,
            4 AS sort_col
 UNION
         SELECT 'tasks_finished'::text AS field_name,
            ((SELECT count(*) FROM monitor_processing WHERE (monitor_processing.status = get_task_status('finished') AND monitor_processing.hostpid = input_hostpid)))::text AS field_value,
            5 AS sort_col
 UNION
         SELECT 'tasks_failed'::text AS field_name,
            ((SELECT count(*) FROM monitor_processing WHERE (monitor_processing.status = get_task_status('failed') AND monitor_processing.hostpid = input_hostpid)))::text AS field_value,
            6 AS sort_col
UNION
		 SELECT 'tasks_running'::text AS field_name,
            ((SELECT count(*) FROM monitor_processing WHERE (monitor_processing.status = get_task_status('started') AND monitor_processing.hostpid = input_hostpid)))::text AS field_value,
            7 AS sort_col
UNION
         SELECT 'tasks_paused'::text AS field_name,
            ((SELECT count(*) FROM monitor_processing WHERE (monitor_processing.status = get_task_status('paused') AND monitor_processing.hostpid = input_hostpid)))::text AS field_value,
            8 AS sort_col
UNION
         SELECT 'tasks_stopped'::text AS field_name,
            ((SELECT count(*) FROM monitor_processing WHERE (monitor_processing.status = get_task_status('stopped') AND monitor_processing.hostpid = input_hostpid)))::text AS field_value,
            9 AS sort_col

UNION
         SELECT 'tasks_waiting'::text AS field_name,
            ((SELECT count(*) FROM monitor_processing WHERE (
	    (monitor_processing.status = get_task_status('none') OR monitor_processing.status IS NULL) AND monitor_processing.hostpid = input_hostpid)))::text AS field_value,
            10 AS sort_col
UNION
         SELECT 'hostpid'::text AS field_name,
            (input_hostpid)::text AS field_value,
            11 AS sort_col
UNION
         SELECT 'total_progress'::text AS field_name,
            (( SELECT calc_host_total_progress(input_hostpid) AS calc_total_progress))::text AS field_value,
            12 AS sort_col
  ORDER BY 3
        )
SELECT stat_raw.field_name,
    stat_raw.field_value
   FROM stat_raw;
END;
$$;
ALTER FUNCTION public.statistics_host(input_hostpid integer) OWNER TO postgres;
--
-- Name: take(integer); Type: FUNCTION; Schema: public; Owner: postgres
-- Zubarev: o_is_mt, mt_coeff needed for exeeded limit memory for JOB
--
CREATE FUNCTION take(agent_id integer, OUT task_id integer, OUT o_session_id integer, OUT o_command text, OUT o_command_args text, OUT o_xml_params text, OUT o_is_mt integer, OUT mt_coeff integer) RETURNS record
    LANGUAGE plpgsql
    AS $$DECLARE task_ids_iterator refcursor;
	task_info RECORD;
	total_parents INT;
	t_agent_id int;
	finished_parents INT;
	id_session INT;
	task_locked bool;
BEGIN
raise notice 'take for agent = %',agent_id;
OPEN task_ids_iterator FOR SELECT * FROM (SELECT DISTINCT ON (tasks_queue.id_task) *
			   FROM tasks_queue
			   WHERE (tasks_queue.status IS NULL
			       OR tasks_queue.status = get_task_status('restarted'))) t
			   ORDER BY t.priority DESC;
fetch next from task_ids_iterator into task_info;
  while found
  loop
     t_agent_id = public.find_apropriate_tasks(task_info.ram_per_cpu_needed, agent_id, task_info.multi_threading);
			IF (t_agent_id IS NULL or t_agent_id<>agent_id) THEN
				    fetch next from task_ids_iterator into task_info;
				CONTINUE;
			END IF;

			IF (agent_has_failed_task(t_agent_id, task_info.id_task)) THEN
				fetch next from task_ids_iterator into task_info;
				CONTINUE;
			END IF;
                        task_locked=pg_try_advisory_lock(1, task_info.id_task);
                        raise notice 'task % locked status %', task_info.id_task, task_locked;
			IF (    task_locked
                                  and
			        (task_info.status IS NULL
				  OR task_info.status = get_task_status('restarted')
                                )
		           ) THEN
				select id, command, command_args, parameters from public.tasks where id=task_info.id_task into task_id, o_command, o_command_args, o_xml_params;
				select public.agents.mt_coeff from public.agents where public.agents.id = agent_id INTO mt_coeff;
				INSERT INTO public.sessions (id_task, id_agent)
				VALUES (task_info.id_task, agent_id) RETURNING public.sessions.id_session INTO o_session_id;
				INSERT INTO public.processing (id_session, progress, status)
				VALUES (o_session_id, 0, get_task_status('started'));
				o_is_mt = task_info.multi_threading;
                                return;
			END IF;
  fetch next from task_ids_iterator into task_info;
  end loop;
  CLOSE task_ids_iterator;
  return;
END;$$;
ALTER FUNCTION public.take(agent_id integer, OUT task_id integer, OUT o_session_id integer, OUT o_command text, OUT o_command_args text, OUT o_xml_params text, OUT o_is_mt integer, OUT mt_coeff integer) OWNER TO postgres;
--
-- Name: unlock_resource(character varying, boolean); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION unlock_resource(file_name character varying, need_exclusive boolean) RETURNS boolean
    LANGUAGE plpgsql IMMUTABLE
    AS $$declare result boolean default FALSE;
	      file_uid int;
begin
	file_uid = public.get_res_id(file_name);
	IF (file_uid IS NOT NULL OR file_uid > 0) THEN
		IF (need_exclusive) THEN
			result = pg_advisory_unlock(0, file_uid);
		ELSE
			result = pg_advisory_unlock_shared(0, file_uid);
		END IF;

	END IF;
	RETURN result;
end;$$;
ALTER FUNCTION public.unlock_resource(file_name character varying, need_exclusive boolean) OWNER TO postgres;
--
-- Name: FUNCTION unlock_resource(file_name character varying, need_exclusive boolean); Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON FUNCTION unlock_resource(file_name character varying, need_exclusive boolean) IS '     ';
--
-- Name: upd_session(); Type: FUNCTION; Schema: public; Owner: postgres
--
CREATE FUNCTION upd_session() RETURNS trigger
    LANGUAGE plpgsql
    AS $$begin
   if(new.finish_time is not null) then
       perform pg_advisory_unlock(1, new.id_task);
   end if;
   return null;
end;$$;
ALTER FUNCTION public.upd_session() OWNER TO postgres;
SET default_tablespace = '';
SET default_with_oids = false;
--
-- Name: agent_perf_data; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE agent_perf_data (
    hostname character varying(128) NOT NULL,
    last_pos integer DEFAULT 0,
    last_update timestamp with time zone DEFAULT now(),
    processor_time double precision[],
    memory_free double precision[],
    disk_time double precision[],
    disk_free_mb double precision[],
    network_bytes_sent double precision[],
    network_bytes_received double precision[]
);
ALTER TABLE agent_perf_data OWNER TO postgres;
--
-- Name: TABLE agent_perf_data; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE agent_perf_data IS 'Agent performance counter values.
Important! If you change the table format, also update the add_perf_data(), clear_perf_data() functions, and the agent_perf_setup_changed trigger.
The table has a cyclic structure (Round-robin). To add data, use the add_perf_data() function.';
--
-- Name: COLUMN agent_perf_data.last_pos; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_perf_data.last_pos IS 'Index of array on which the counter values were last written';
--
-- Name: COLUMN agent_perf_data.last_update; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_perf_data.last_update IS 'Time when counter values were last written';
--
-- Name: agent_perf_setup; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE agent_perf_setup (
    frequency double precision,
    history_len integer
);
ALTER TABLE agent_perf_setup OWNER TO postgres;
--
-- Name: TABLE agent_perf_setup; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE agent_perf_setup IS 'Settings for obtaining and storing data of the performance of agents.
Changes to the values in this table will remove all data from the agent_perf_data table.';
--
-- Name: COLUMN agent_perf_setup.frequency; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_perf_setup.frequency IS 'Frequency with which agent will send performance counter values. ';
--
-- Name: COLUMN agent_perf_setup.history_len; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_perf_setup.history_len IS 'Maximum size of the history (ring buffer) of stored measurements in the agent_perf_data table.';
--
-- Name: agent_performance_counters; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE agent_performance_counters (
    agent_id integer DEFAULT 0 NOT NULL,
    counter_name character varying(128) NOT NULL,
    counter_value double precision DEFAULT 0,
    counter_path character varying(128) NOT NULL,
    measurement_time timestamp with time zone DEFAULT now() NOT NULL,
    id bigint NOT NULL,
    group_id bigint NOT NULL
);
ALTER TABLE agent_performance_counters OWNER TO postgres;
--
-- Name: TABLE agent_performance_counters; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE agent_performance_counters IS 'Contains performance counters for each agent.
https://msdn.microsoft.com/en-us/library/aa939698(v=winembedded.5).aspx';
--
-- Name: COLUMN agent_performance_counters.agent_id; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_performance_counters.agent_id IS 'Agent ID';
--
-- Name: COLUMN agent_performance_counters.counter_name; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_performance_counters.counter_name IS 'Counter Name';
--
-- Name: COLUMN agent_performance_counters.counter_value; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_performance_counters.counter_value IS 'Counter value';
--
-- Name: COLUMN agent_performance_counters.counter_path; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_performance_counters.counter_path IS 'Path to the counter in the registry';
--
-- Name: COLUMN agent_performance_counters.measurement_time; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_performance_counters.measurement_time IS 'Measuring time';
--
-- Name: COLUMN agent_performance_counters.group_id; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agent_performance_counters.group_id IS 'Measurement group identifier for data processing convenience.';
--
-- Name: agent_performance_counters_group_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE agent_performance_counters_group_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE agent_performance_counters_group_id_seq OWNER TO postgres;
--
-- Name: agent_performance_counters_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE agent_performance_counters_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE agent_performance_counters_id_seq OWNER TO postgres;
--
-- Name: agent_performance_counters_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--
ALTER SEQUENCE agent_performance_counters_id_seq OWNED BY agent_performance_counters.id;
--
-- Name: agents_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE agents_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE agents_id_seq OWNER TO postgres;
--
-- Name: agents; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE agents (
    id integer DEFAULT nextval('agents_id_seq'::regclass) NOT NULL,
    hostname character varying(128) NOT NULL,
    cpu_count integer NOT NULL,
    ram_size bigint NOT NULL,
    last_update_time timestamp with time zone,
    fake_cpu_count integer,
    mt_coeff integer DEFAULT 1,
    is_paused boolean DEFAULT false NOT NULL,
    ip_address character varying,
    version character varying,
    is_alive boolean DEFAULT true NOT NULL,
	max_cpu_count integer DEFAULT 20 NOT NULL,
	gpu_memory vram_item,
	avail_gpu_devices vram_item[]
);
ALTER TABLE agents OWNER TO postgres;
--
-- Name: TABLE agents; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE agents IS 'The table containing data about task executing agents.';
--
-- Name: COLUMN agents.id; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agents.id IS 'Unique agent identifier';
--
-- Name: COLUMN agents.hostname; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agents.hostname IS 'Name of the agent in the network';
--
-- Name: COLUMN agents.cpu_count; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agents.cpu_count IS 'Number of processor cores (physical and virtual)';
--
-- Name: COLUMN agents.ram_size; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agents.ram_size IS 'Amount of RAM in bytes';
--
-- Name: COLUMN agents.last_update_time; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN agents.last_update_time IS 'Agents life indicator. The alive agent constantly updates this field.';
--
-- Name: agents_alive; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW agents_alive WITH (security_barrier='false') AS
 SELECT agents.id,
    agents.hostname,
        CASE (agents.fake_cpu_count IS NOT NULL)
            WHEN true THEN agents.fake_cpu_count
            ELSE agents.cpu_count
        END AS cpu_count,
    agents.ram_size,
    agents.last_update_time,
        CASE agent_has_launched_mt_tasks(agents.id)
            WHEN true THEN (count_launched_tasks(agents.id) * (floor(((agents.fake_cpu_count / agents.mt_coeff))::double precision))::integer)
            ELSE count_launched_tasks(agents.id)
        END AS task_count,
        CASE (agents.mt_coeff IS NOT NULL)
            WHEN true THEN agents.mt_coeff
            ELSE agents.cpu_count
        END AS mt_coeff,
    agents.is_paused AS status,
    agents.ip_address,
    agents.version,
	agents.cpu_count AS core_quantity,
	agents.gpu_memory,
	agents.avail_gpu_devices
   FROM agents
  WHERE ((( SELECT now() AS now) - agents.last_update_time) < '00:00:10'::interval);
ALTER TABLE agents_alive OWNER TO postgres;
--
-- Name: VIEW agents_alive; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON VIEW agents_alive IS 'View of the Agents table that filters out all dead agents.';
--
-- Name: agents_alive_info; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW agents_alive_info AS
 SELECT agents_alive.id,
    agents_alive.hostname,
    agents_alive.cpu_count AS max_tasks,
    agents_alive.core_quantity,
    agents_alive.ram_size,
    agents_alive.last_update_time,
    public.count_launched_tasks(agents_alive.id, true) AS current_tasks,
    agents_alive.mt_coeff AS max_mt_tasks,
    agents_alive.status,
    agents_alive.ip_address,
    agents_alive.version,
	agents_alive.gpu_memory,
	agents_alive.avail_gpu_devices
   FROM agents_alive;
ALTER TABLE agents_alive_info OWNER TO postgres;
--
-- Name: agents_alive_task_count; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW agents_alive_task_count WITH (security_barrier='false') AS
 SELECT agents_alive.id,
    agents_alive.cpu_count,
    agents_alive.ram_size,
    agents_alive.task_count,
    (agents_alive.ram_size - calc_used_ram_size(agents_alive.id)) AS ram_size_left,
    agents_alive.mt_coeff,
    agents_alive.status AS is_paused
   FROM agents_alive;
ALTER TABLE agents_alive_task_count OWNER TO postgres;
--
-- Name: VIEW agents_alive_task_count; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON VIEW agents_alive_task_count IS 'View of the agents and processing table showing all alive agents and the number of tasks running (excluding completed) on them. ';
--
-- Name: sessions_id_session_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE sessions_id_session_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE sessions_id_session_seq OWNER TO postgres;
--
-- Name: sessions; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE sessions (
    id_session integer DEFAULT nextval('sessions_id_session_seq'::regclass) NOT NULL,
    id_task integer NOT NULL,
    id_agent integer NOT NULL,
    start_time timestamp with time zone DEFAULT now() NOT NULL,
    elapsed_time interval DEFAULT '00:00:00'::interval NOT NULL,
    finish_time timestamp with time zone,
	has_warnings boolean DEFAULT false NOT NULL
);
ALTER TABLE sessions OWNER TO postgres;
--
-- Name: TABLE sessions; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE sessions IS 'Table containing information about sessions. Each running task is assigned, in addition to the identifier, also a session identifier to be able to create a new log after restarting the completed task.';
--
-- Name: COLUMN sessions.id_session; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN sessions.id_session IS 'The ID of the current session. Necessary in order to be able to create a new log after restarting a completed task.';
--
-- Name: COLUMN sessions.id_task; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN sessions.id_task IS 'Task ID';
--
-- Name: COLUMN sessions.start_time; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN sessions.start_time IS 'Session start time';
--
-- Name: last_sessions; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW last_sessions WITH (security_barrier='true') AS
 WITH last_sessions AS (
         SELECT DISTINCT ON ((max(sessions.id_session))) max(sessions.id_session) AS id_session,
            sessions.id_task
           FROM sessions
          GROUP BY sessions.id_task
        )
 SELECT s.id_session,
    s.id_task,
    s.id_agent,
    s.start_time,
    s.elapsed_time,
    s.finish_time,
	s.has_warnings
   FROM (sessions s
     JOIN last_sessions ON ((last_sessions.id_session = s.id_session)));
ALTER TABLE last_sessions OWNER TO postgres;
--
-- Name: processing_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE processing_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE processing_id_seq OWNER TO postgres;
--
-- Name: processing; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE processing (
    id integer DEFAULT nextval('processing_id_seq'::regclass) NOT NULL,
    id_session integer NOT NULL,
    progress integer NOT NULL,
    status integer,
    update_time timestamp with time zone DEFAULT now() NOT NULL
);
ALTER TABLE processing OWNER TO postgres;
--
-- Name: TABLE processing; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE processing IS 'Table containing information about tasks that are in progress or have already been completed.';
--
-- Name: COLUMN processing.id; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN processing.id IS 'row unique identifier';
--
-- Name: COLUMN processing.id_session; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN processing.id_session IS 'running session ID';
--
-- Name: COLUMN processing.progress; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN processing.progress IS 'task progress (0-100%)';
--
-- Name: COLUMN processing.status; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN processing.status IS 'Task status. See table task_status_description
';
--
-- Name: session_status; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW session_status AS
 SELECT DISTINCT ON (last_sessions.id_session) last_sessions.id_session,
    last_sessions.id_task,
    last_sessions.id_agent,
    processing.status,
    processing.progress,
    last_sessions.start_time,
    last_sessions.elapsed_time,
    processing.update_time,
	last_sessions.has_warnings
   FROM (last_sessions
     LEFT JOIN processing ON ((last_sessions.id_session = processing.id_session)));
ALTER TABLE session_status OWNER TO postgres;
--
-- Name: tasks; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE tasks (
    id integer NOT NULL,
    name character varying NOT NULL,
    block_description character varying,
    ram_per_cpu_needed bigint NOT NULL,
    parameters text NOT NULL,
    registration_time timestamp with time zone DEFAULT now() NOT NULL,
    hostname character varying(256) NOT NULL,
    hostpid bigint NOT NULL,
    command character varying(256),
    command_args character varying(256),
    multi_threading integer DEFAULT 0 NOT NULL,
    priority integer DEFAULT 0 NOT NULL,
    manual_start boolean DEFAULT false NOT NULL
);
ALTER TABLE tasks OWNER TO postgres;
--
-- Name: TABLE tasks; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE tasks IS 'Table containing data about the assigned tasks.';
--
-- Name: COLUMN tasks.id; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.id IS 'Unique task ID';
--
-- Name: COLUMN tasks.name; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.name IS 'Task name';
--
-- Name: COLUMN tasks.ram_per_cpu_needed; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.ram_per_cpu_needed IS 'The ratio of the amount of RAM required for this task per processor core.';
--
-- Name: COLUMN tasks.parameters; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.parameters IS 'Internal task parameters passed as xml.';
--
-- Name: COLUMN tasks.registration_time; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.registration_time IS 'Time of task queuing for processing';
--
-- Name: COLUMN tasks.hostname; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.hostname IS 'The name of the computer that submitted the task.';
--
-- Name: COLUMN tasks.hostpid; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.hostpid IS 'The ID of the process that submitted the task.';
--
-- Name: COLUMN tasks.command; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.command IS 'The name of the exe to run.';
--
-- Name: COLUMN tasks.command_args; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.command_args IS 'Command line options to run.';
--
-- Name: COLUMN tasks.multi_threading; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN tasks.multi_threading IS 'Indicates that the task requires multiple threads to execute. 1 - required, 0 - not required.';
--
-- Name: agents_alive_task_manager; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW agents_alive_task_manager AS
 SELECT ss.id_task,
    ss.id_agent,
    ss.status,
    t.ram_per_cpu_needed,
    t.multi_threading
   FROM ((session_status ss
     LEFT JOIN tasks t ON ((t.id = ss.id_task)))
     JOIN agents a ON (((ss.id_agent = a.id) AND ((( SELECT now() AS now) - a.last_update_time) < '00:00:10'::interval))));
ALTER TABLE agents_alive_task_manager OWNER TO postgres;
--
-- Name: agents_main_perf_counters; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW agents_main_perf_counters AS
 WITH main_perf_ctrs AS (
         SELECT ct.id_agent,
            ct.hdd_usage_pcent,
            ct.hdd_free_megabytes,
            ct.ram_free_bytes,
            ct.cpu_usage_pcent
           FROM crosstab('select agent_id, counter_name, counter_value
	   from last_performance_measurements
	   where counter_name = ''Disk Time %''
	   or counter_name = ''Free Disk Megabytes''
	   or counter_name = ''Free Memory Bytes''
	   or counter_name = ''Processor Time %''
	   order by 1,2'::text) ct(id_agent integer, hdd_usage_pcent double precision, hdd_free_megabytes double precision, ram_free_bytes double precision, cpu_usage_pcent double precision)
        )
 SELECT main_perf_ctrs.id_agent,
    main_perf_ctrs.cpu_usage_pcent,
    main_perf_ctrs.ram_free_bytes,
    main_perf_ctrs.hdd_free_megabytes,
    main_perf_ctrs.hdd_usage_pcent
   FROM main_perf_ctrs;
ALTER TABLE agents_main_perf_counters OWNER TO postgres;
--
-- Name: VIEW agents_main_perf_counters; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON VIEW agents_main_perf_counters IS 'Key performance indicators of each agent';
--
-- Name: agents_perf; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW agents_perf WITH (security_barrier='false') AS
 WITH perf AS (
         SELECT 'Overall Performance'::text AS hostname,
            max(p.last_update) AS last_update,
            avg(p.processor_time[p.last_pos]) AS processor_time,
            avg(p.memory_free[p.last_pos]) AS memory_free,
            avg(p.disk_time[p.last_pos]) AS disk_time,
            avg(p.disk_free_mb[p.last_pos]) AS disk_free_mb,
            avg(p.network_bytes_sent[p.last_pos]) AS network_bytes_sent,
            avg(p.network_bytes_received[p.last_pos]) AS network_bytes_received
           FROM agent_perf_data p
        UNION ALL
         SELECT p.hostname,
            p.last_update,
            p.processor_time[p.last_pos] AS processor_time,
            p.memory_free[p.last_pos] AS memory_free,
            p.disk_time[p.last_pos] AS disk_time,
            p.disk_free_mb[p.last_pos] AS disk_free_mb,
            p.network_bytes_sent[p.last_pos] AS network_bytes_sent,
            p.network_bytes_received[p.last_pos] AS network_bytes_received
           FROM agent_perf_data p
        )
 SELECT perf.hostname,
    perf.last_update,
    perf.processor_time,
    perf.memory_free,
    perf.disk_time,
    perf.disk_free_mb,
    perf.network_bytes_sent,
    perf.network_bytes_received
   FROM perf
  WHERE (perf.last_update IS NOT NULL);
ALTER TABLE agents_perf OWNER TO postgres;
--
-- Name: agents_task_count; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW agents_task_count AS
 SELECT agents.id,
    agents.cpu_count,
    agents.ram_size,
        CASE agent_has_launched_mt_tasks(agents.id)
            WHEN true THEN agents.cpu_count
            ELSE count_launched_tasks(agents.id)
        END AS task_count,
    (agents.ram_size - calc_used_ram_size(agents.id)) AS ram_size_left
   FROM agents;
ALTER TABLE agents_task_count OWNER TO postgres;
--
-- Name: avg_group_performance_measurements; Type: MATERIALIZED VIEW; Schema: public; Owner: postgres
--
CREATE MATERIALIZED VIEW avg_group_performance_measurements AS
 WITH main_perf_ctrs AS (
         SELECT ct.group_id,
            ct.hdd_usage_pcent,
            ct.hdd_free_megabytes,
            ct.ram_free_bytes,
            ct.cpu_usage_pcent
           FROM crosstab('select group_id, counter_name, counter_value
	   from agent_performance_counters
	   where counter_name = ''Disk Time %''
	   or counter_name = ''Free Disk Megabytes''
	   or counter_name = ''Free Memory Bytes''
	   or counter_name = ''Processor Time %''
	   order by 1,2'::text) ct(group_id bigint, hdd_usage_pcent double precision, hdd_free_megabytes double precision, ram_free_bytes double precision, cpu_usage_pcent double precision)
        )
 SELECT main_perf_ctrs.group_id,
    main_perf_ctrs.cpu_usage_pcent,
    main_perf_ctrs.ram_free_bytes,
    main_perf_ctrs.hdd_free_megabytes,
    main_perf_ctrs.hdd_usage_pcent,
    ( SELECT ((avg(date_part('epoch'::text, apc.measurement_time)) * '00:00:01'::interval) + '1970-01-01 03:00:00+03'::timestamp with time zone)
           FROM agent_performance_counters apc
          WHERE (main_perf_ctrs.group_id = apc.group_id)
          GROUP BY apc.group_id) AS avg_measurement_time,
    ( SELECT apc.agent_id
           FROM agent_performance_counters apc
          WHERE (apc.group_id = main_perf_ctrs.group_id)
         LIMIT 1) AS id_agent
   FROM main_perf_ctrs
  WITH NO DATA;
ALTER TABLE avg_group_performance_measurements OWNER TO postgres;
--
-- Name: command_type_description; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE command_type_description (
    command_name character varying(128) NOT NULL,
    id_command integer NOT NULL
);
ALTER TABLE command_type_description OWNER TO postgres;
--
-- Name: monitor_processing; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW monitor_processing WITH (security_barrier='false') AS
 SELECT tasks.name,
    tasks.block_description,
    session_status.progress,
    session_status.status,
    session_status.start_time,
    session_status.elapsed_time,
    tasks.id AS id_task,
    session_status.id_session,
    session_status.id_agent,
    tasks.hostname,
    tasks.hostpid,
    tasks.multi_threading,
    session_status.update_time,
    tasks.priority,
	session_status.has_warnings
   FROM (tasks
     LEFT JOIN session_status ON ((tasks.id = session_status.id_task)));
ALTER TABLE monitor_processing OWNER TO postgres;
--
-- Name: task_dependencies_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE task_dependencies_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE task_dependencies_id_seq OWNER TO postgres;
--
-- Name: task_dependencies; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE task_dependencies (
    id integer DEFAULT nextval('task_dependencies_id_seq'::regclass) NOT NULL,
    id_group bigint,
    id_dependency bigint,
    id_task integer
);
ALTER TABLE task_dependencies OWNER TO postgres;
--
-- Name: TABLE task_dependencies; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE task_dependencies IS 'Table containing task dependencies, as a pair of identifiers.';
--
-- Name: COLUMN task_dependencies.id_group; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN task_dependencies.id_group IS 'ID of the group that the task belongs to (a unique number generated by client)';
--
-- Name: COLUMN task_dependencies.id_dependency; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN task_dependencies.id_dependency IS 'ID of the task group on which the current task depends';
--
-- Name: COLUMN task_dependencies.id_task; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN task_dependencies.id_task IS 'Task ID';
--
-- Name: groups_finished; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW groups_finished AS
SELECT td.id_group,
    td.id_dependency
   FROM task_dependencies td
     JOIN monitor_processing mp ON td.id_task = mp.id_task AND mp.status = 3
EXCEPT
 SELECT td.id_group,
    td.id_dependency
   FROM task_dependencies td
  WHERE (td.id_group IN ( SELECT td1.id_group
           FROM task_dependencies td1
             JOIN monitor_processing mp ON td1.id_task = mp.id_task AND mp.status IS DISTINCT FROM 3));
ALTER TABLE groups_finished OWNER TO postgres;
--
-- Name: current_phase_tasks; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW current_phase_tasks WITH (security_barrier='false') AS
 SELECT DISTINCT ON (td.id_task) td.id_task,
    mp.status,
    mp.progress,
    mp.start_time
   FROM ((task_dependencies td
     JOIN ( SELECT td1.id_group
           FROM ((task_dependencies td1
             JOIN groups_finished tt ON (((td1.id_dependency = tt.id_group) OR (td1.id_dependency = 0))))
             JOIN monitor_processing mp1 ON (((td1.id_task = mp1.id_task) AND ((mp1.status = 1) OR (mp1.status = 7) OR (mp1.status = 9)))))) z ON ((td.id_group = z.id_group)))
     JOIN monitor_processing mp ON ((td.id_task = mp.id_task)));
ALTER TABLE current_phase_tasks OWNER TO postgres;
--
-- Name: failed_tasks; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE failed_tasks (
    id_task integer NOT NULL,
    id_agent integer NOT NULL
);
ALTER TABLE failed_tasks OWNER TO postgres;
--
-- Name: hosts; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW hosts AS
 SELECT monitor_processing.hostname AS maker_name,
    monitor_processing.hostpid,
    min(monitor_processing.start_time) AS start_time
   FROM monitor_processing
  GROUP BY monitor_processing.hostpid, monitor_processing.hostname;
ALTER TABLE hosts OWNER TO postgres;
--
-- Name: last_performance_measurements; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW last_performance_measurements AS
 SELECT left_apc.agent_id,
    left_apc.counter_name,
    left_apc.counter_path,
    left_apc.counter_value,
    right_apc.last_measurement_time
   FROM (agent_performance_counters left_apc
     JOIN ( SELECT agent_performance_counters.agent_id,
            agent_performance_counters.counter_path,
            max(agent_performance_counters.measurement_time) AS last_measurement_time
           FROM agent_performance_counters
          GROUP BY agent_performance_counters.agent_id, agent_performance_counters.counter_path) right_apc ON (((left_apc.agent_id = right_apc.agent_id) AND ((left_apc.counter_path)::text = (right_apc.counter_path)::text) AND (left_apc.measurement_time = right_apc.last_measurement_time))));
ALTER TABLE last_performance_measurements OWNER TO postgres;
--
-- Name: log; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE log (
    id integer NOT NULL,
    message text NOT NULL,
    id_session integer NOT NULL,
    msg_level integer DEFAULT 3 NOT NULL,
    module_name character varying(256),
    log_time timestamp with time zone DEFAULT now() NOT NULL,
    msg_type character varying(16) DEFAULT 'I'::bpchar NOT NULL
);
ALTER TABLE log OWNER TO postgres;
--
-- Name: TABLE log; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE log IS 'Table for storing task execution logs.';
--
-- Name: COLUMN log.id; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN log.id IS 'UID';
--
-- Name: COLUMN log.message; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN log.message IS 'Message text';
--
-- Name: COLUMN log.id_session; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN log.id_session IS 'Session ID';
--
-- Name: COLUMN log.msg_level; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN log.msg_level IS 'Message severity level
  phllCritical=0, //!< Critical: in theory, should always be shown, wait for OK
   phllImportant=1, //!< Important, should be shown, progress log window will wait OK
   phllHigher=2, //!< More important than usual. Used in subprogress
   phllNormal=3, //!< Normal level. Suitable for most occasions
   phllLower=4, //!< Less important than usual. Used in subprogress
   phllTrace=5, //!< Debug message, not displayed to the user';
--
-- Name: COLUMN log.log_time; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN log.log_time IS 'Message creation time';
--
-- Name: COLUMN log.msg_type; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN log.msg_type IS 'Message type. I - debug, W - warning, E - error';
--
-- Name: log_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE log_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE log_id_seq OWNER TO postgres;
--
-- Name: log_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--
ALTER SEQUENCE log_id_seq OWNED BY log.id;
--
-- Name: log_performance; Type: MATERIALIZED VIEW; Schema: public; Owner: postgres
--
CREATE MATERIALIZED VIEW log_performance AS
 WITH agent_session AS (
         SELECT log.id,
            log.message,
            log.id_session,
            log.msg_level,
            log.module_name,
            log.log_time,
            log.msg_type,
            ss.id_agent
           FROM (log
             LEFT JOIN sessions ss USING (id_session))
        ), closest_measurement_timestamp AS (
         SELECT agsess.id_agent,
            agsess.message,
            agsess.log_time,
            ( SELECT agpm_1.group_id
                   FROM avg_group_performance_measurements agpm_1
                  WHERE (agpm_1.id_agent = agsess.id_agent)
                  ORDER BY GREATEST((agpm_1.avg_measurement_time - agsess.log_time), (agsess.log_time - agpm_1.avg_measurement_time))
                 LIMIT 1) AS group_id,
            agsess.msg_type,
            agsess.id_session,
            agsess.id
           FROM agent_session agsess
        )
 SELECT cmt.id,
    cmt.id_agent,
    cmt.message,
    cmt.log_time,
    agpm.avg_measurement_time,
    agpm.cpu_usage_pcent,
    agpm.ram_free_bytes,
    agpm.hdd_free_megabytes,
    agpm.hdd_usage_pcent,
    agpm.group_id,
    cmt.msg_type,
    cmt.id_session
   FROM (closest_measurement_timestamp cmt
     JOIN avg_group_performance_measurements agpm ON ((cmt.group_id = agpm.group_id)))
  ORDER BY cmt.group_id
  WITH NO DATA;
ALTER TABLE log_performance OWNER TO postgres;
--
-- Name: MATERIALIZED VIEW log_performance; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON MATERIALIZED VIEW log_performance IS 'View that combines log events with corresponding performance measurement events';
--
-- Name: task_dependencies_info; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW task_dependencies_info AS
SELECT td1.id_task, 
    td1.id_group, 
    array_agg(td1.id_dependency) AS id_dependency
   FROM task_dependencies td1
   GROUP BY td1.id_task, td1.id_group
   ORDER BY td1.id_task, td1.id_group;
ALTER TABLE task_dependencies_info OWNER TO postgres;
--
-- Name: monitor_agents_alive_main_perf_counters; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW monitor_agents_alive_main_perf_counters AS
 SELECT aa.id,
    aa.hostname,
    aa.cpu_count,
    ampc.cpu_usage_pcent,
    ampc.ram_free_bytes,
    ampc.hdd_usage_pcent,
    ampc.hdd_free_megabytes,
    aa.last_update_time
   FROM (agents_alive aa
     LEFT JOIN agents_main_perf_counters ampc ON ((aa.id = ampc.id_agent)));
ALTER TABLE monitor_agents_alive_main_perf_counters OWNER TO postgres;
--
-- Name: monitor_hosts; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW monitor_hosts AS
  SELECT 'All'::character varying AS maker_name,
    NULL::bigint AS hostpid,
    min(hosts.start_time) AS start_time
   FROM hosts
UNION ALL
 SELECT hosts.maker_name,
    hosts.hostpid,
    hosts.start_time
   FROM hosts;
ALTER TABLE monitor_hosts OWNER TO postgres;
--
-- Name: monitor_processing_info; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW monitor_processing_info AS
 SELECT monitor_processing.name,
    monitor_processing.progress,
    monitor_processing.status,
    monitor_processing.start_time AS started_at,
    monitor_processing.elapsed_time,
    monitor_processing.id_task,
    monitor_processing.id_session,
    monitor_processing.id_agent,
    monitor_processing.hostname AS maker_name,
    monitor_processing.hostpid,
    monitor_processing.multi_threading,
    monitor_processing.update_time,
    monitor_processing.priority,
    agents.hostname AS executor,
    tasks.registration_time AS created_at,
    tasks.command_args AS profile,
    monitor_processing.block_description,
	monitor_processing.has_warnings,
	task_dependencies_info.id_group,
	task_dependencies_info.id_dependency
   FROM monitor_processing
     LEFT JOIN agents ON monitor_processing.id_agent = agents.id
	 LEFT JOIN task_dependencies_info ON monitor_processing.id_task = task_dependencies_info.id_task
     LEFT JOIN tasks ON monitor_processing.id_task = tasks.id;
ALTER TABLE monitor_processing_info OWNER TO postgres;
--
-- Name: task_parents; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW task_parents AS
SELECT td1.id_task,
    td2.id_task AS id_parent,
    td1.id_dependency AS parent_group
   FROM task_dependencies td1
     LEFT JOIN task_dependencies td2 ON td1.id_dependency = td2.id_group
  WHERE td1.id_dependency <> 0 AND td1.id_dependency IS NOT NULL;
ALTER TABLE task_parents OWNER TO postgres;
--
-- Name: monitor_task_graph; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW monitor_task_graph AS
 SELECT tasks.id,
    task_parents.id_parent,
    tasks.name,
    session_status.status,
    tasks.hostname,
    tasks.hostpid
   FROM ((tasks
     LEFT JOIN task_parents ON ((tasks.id = task_parents.id_task)))
     LEFT JOIN session_status ON ((tasks.id = session_status.id_task)));
ALTER TABLE monitor_task_graph OWNER TO postgres;
--
-- Name: VIEW monitor_task_graph; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON VIEW monitor_task_graph IS 'View used to display the task graph.';
--
-- Name: processing_parameters; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE processing_parameters (
    manual_mode boolean DEFAULT false NOT NULL,
    db_version integer NOT NULL,
	db_version_sha character varying(512) DEFAULT 0 NOT NULL
);
ALTER TABLE processing_parameters OWNER TO postgres;
--
-- Name: resource_identificators; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE resource_identificators (
    res_name character varying(512) NOT NULL,
    id integer NOT NULL,
    status character varying(512)
);
ALTER TABLE resource_identificators OWNER TO postgres;
--
-- Name: TABLE resource_identificators; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE resource_identificators IS 'Table that specifies a unique identifier for the file name. This identifier is used to block files in the resource system using the pg_advisory_block(int,int) function';
--
-- Name: resource_identificators_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE resource_identificators_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE resource_identificators_id_seq OWNER TO postgres;
--
-- Name: resource_identificators_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--
ALTER SEQUENCE resource_identificators_id_seq OWNED BY resource_identificators.id;

--
-- Name: task_status; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW task_status WITH (security_barrier='false') AS
 SELECT tasks.id AS id_task,
    task_parents.id_parent,
    session_status.status,
    tasks.ram_per_cpu_needed,
    tasks.name,
    tasks.multi_threading,
    tasks.priority,
    task_dependencies.id_dependency
   FROM (((tasks
     LEFT JOIN session_status ON ((tasks.id = session_status.id_task)))
     LEFT JOIN task_parents ON ((tasks.id = task_parents.id_task)))
     LEFT JOIN task_dependencies ON ((tasks.id = task_dependencies.id_task)))
  WHERE ((is_manual_mode() AND tasks.manual_start) OR (NOT is_manual_mode()))
  ORDER BY task_parents.id_parent NULLS FIRST;
ALTER TABLE task_status OWNER TO postgres;
--
-- Name: VIEW task_status; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON VIEW task_status IS 'View that displays the tasks dependency, status, and characteristics.';
--
-- Name: statistics_tasks_status; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW statistics_tasks_status WITH (security_barrier='false') AS
 SELECT ( SELECT count(*) AS count
           FROM tasks) AS tasks_total,
    ( SELECT count(DISTINCT task_status.id_task) AS count
           FROM task_status
          WHERE (task_status.status = get_task_status('finished'::character varying))) AS tasks_finished,
    ( SELECT count(DISTINCT task_status.id_task) AS count
           FROM task_status
          WHERE (task_status.status = get_task_status('failed'::character varying))) AS tasks_failed,
    ( SELECT count(DISTINCT task_status.id_task) AS count
           FROM task_status
          WHERE (task_status.status = get_task_status('started'::character varying))) AS tasks_running,
    ( SELECT count(DISTINCT task_status.id_task) AS count
           FROM task_status
          WHERE (task_status.status = get_task_status('paused'::character varying))) AS tasks_paused,
    ( SELECT count(DISTINCT task_status.id_task) AS count
           FROM task_status
          WHERE (task_status.status IS NULL)) AS tasks_waiting;
ALTER TABLE statistics_tasks_status OWNER TO postgres;
--
-- Name: statistics_time; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW statistics_time AS
 WITH time_stat AS (
         SELECT min(sessions.start_time) AS min_start_time,
            max(GREATEST(sessions.finish_time, (sessions.start_time + sessions.elapsed_time))) AS max_finish_time
           FROM sessions
        )
 SELECT time_stat.min_start_time,
    time_stat.max_finish_time,
    (time_stat.max_finish_time - time_stat.min_start_time) AS total_elapsed_time
   FROM time_stat;
ALTER TABLE statistics_time OWNER TO postgres;
--
-- Name: statistics_total; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW statistics_total WITH (security_barrier='false') AS
 WITH stat_raw AS (
         SELECT 'start_time'::text AS field_name,
            (( SELECT min(monitor_processing.start_time) AS min
                   FROM monitor_processing))::text AS field_value,
            1 AS sort_col
        UNION
         SELECT 'finish_time'::text AS field_name,
            (( SELECT max(GREATEST(sessions.finish_time, monitor_processing.start_time + monitor_processing.elapsed_time)) AS max
                   FROM monitor_processing
                     JOIN sessions ON monitor_processing.id_task = sessions.id_task))::text AS field_value,
            2 AS sort_col
        UNION
           SELECT 'elapsed_time'::text AS field_name,
            (( SELECT max(GREATEST(sessions.finish_time, monitor_processing.start_time + monitor_processing.elapsed_time)) - min(monitor_processing.start_time) AS sum
                   FROM monitor_processing
                     JOIN sessions ON monitor_processing.id_task = sessions.id_task))::text AS field_value,
            3 AS sort_col
        UNION
         SELECT 'tasks_total'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM monitor_processing))::text AS field_value,
            4 AS sort_col
        UNION
         SELECT 'tasks_finished'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM monitor_processing
                  WHERE monitor_processing.status = get_task_status('finished'::character varying)))::text AS field_value,
            5 AS sort_col
        UNION
         SELECT 'tasks_failed'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM monitor_processing
                  WHERE monitor_processing.status = get_task_status('failed'::character varying) OR monitor_processing.status = get_task_status('semifailed'::character varying)))::text AS field_value,
            6 AS sort_col
        UNION
         SELECT 'tasks_running'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM monitor_processing
                  WHERE monitor_processing.status = get_task_status('started'::character varying)))::text AS field_value,
            7 AS sort_col
        UNION
         SELECT 'tasks_paused'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM monitor_processing
                  WHERE monitor_processing.status = get_task_status('paused'::character varying)))::text AS field_value,
            8 AS sort_col
		 UNION
         SELECT 'tasks_stopped'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM monitor_processing
                  WHERE monitor_processing.status = get_task_status('stopped'::character varying)))::text AS field_value,
            9 AS sort_col
        UNION
         SELECT 'tasks_waiting'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM monitor_processing
                  WHERE monitor_processing.status = get_task_status('none'::character varying) OR monitor_processing.status IS NULL))::text AS field_value,
            10 AS sort_col
        UNION
         SELECT 'active_agents'::text AS field_name,
            (( SELECT count(*) AS count
                   FROM agents_alive))::text AS field_value,
            11 AS sort_col
        UNION
        SELECT 'total_cpu_count'::text AS field_name,
            (( SELECT sum(agents_alive.core_quantity) AS total_cpu_count
                   FROM agents_alive))::text AS field_value,
            12 AS sort_col
        UNION
         SELECT 'total_progress'::text AS field_name,
            (( SELECT calc_total_progress() AS calc_total_progress))::text AS field_value,
            13 AS sort_col
  ORDER BY 3
        )
 SELECT stat_raw.field_name,
    stat_raw.field_value
   FROM stat_raw;
ALTER TABLE statistics_total OWNER TO postgres;
--
-- Name: task_parents_status; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW task_parents_status AS
 SELECT t1.id AS id_task,
    p1.status AS task_status,
    t1.name AS task_name,
    tp.id_parent,
    p2.status AS parent_status,
    t2.name AS parent_name,
    tp.parent_group
   FROM tasks t1
     LEFT JOIN session_status p1 ON p1.id_task = t1.id
     LEFT JOIN task_parents tp ON tp.id_task = t1.id
     LEFT JOIN session_status p2 ON p2.id_task = tp.id_parent
     LEFT JOIN tasks t2 ON tp.id_parent = t2.id;
ALTER TABLE task_parents_status OWNER TO postgres;
--
-- Name: VIEW task_parents_status; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON VIEW task_parents_status IS 'View that displays the status of tasks and their all parents.';
--
-- Name: task_status_description; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE task_status_description (
    id integer NOT NULL,
    status_name character varying(128) NOT NULL
);
ALTER TABLE task_status_description OWNER TO postgres;
--
-- Name: COLUMN task_status_description.status_name; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN task_status_description.status_name IS 'Task status name';
--
-- Name: tasks_blocks; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW tasks_blocks WITH (security_barrier='false') AS
 SELECT monitor_processing.block_description,
    min(monitor_processing.start_time) AS start_time,
    max(monitor_processing.update_time) AS update_time,
    (max(monitor_processing.update_time) - min(monitor_processing.start_time)) AS elapsed_time,
    ( SELECT agregate_progress_and_status.out_progress
           FROM agregate_progress_and_status(monitor_processing.block_description) agregate_progress_and_status(out_progress, out_status)) AS progress,
    ( SELECT agregate_progress_and_status.out_status
           FROM agregate_progress_and_status(monitor_processing.block_description) agregate_progress_and_status(out_progress, out_status)) AS status
   FROM monitor_processing
  GROUP BY monitor_processing.block_description;
ALTER TABLE tasks_blocks OWNER TO postgres;
--
-- Name: tasks_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE tasks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE tasks_id_seq OWNER TO postgres;
--
-- Name: tasks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--
ALTER SEQUENCE tasks_id_seq OWNED BY tasks.id;
--
-- Name: tasks_queue; Type: VIEW; Schema: public; Owner: postgres
--
CREATE VIEW tasks_queue WITH (security_barrier='false') AS
 WITH ps AS (
         SELECT task_parents_status.id_task
           FROM task_parents_status
          WHERE task_parents_status.id_parent IS NOT NULL AND task_parents_status.parent_status IS DISTINCT FROM 3
		  OR task_parents_status.id_parent IS NULL AND task_parents_status.parent_group <> 0 AND task_parents_status.parent_group IS NOT NULL
        )
 SELECT task_status.id_task,
    task_status.id_parent,
    task_status.status,
    task_status.ram_per_cpu_needed,
    task_status.name,
    task_status.multi_threading,
    task_status.priority,
    task_status.id_dependency
   FROM (public.task_status
     LEFT JOIN ps ON ((task_status.id_task = ps.id_task)))
  WHERE (ps.id_task IS NULL);
ALTER TABLE tasks_queue OWNER TO postgres;
--
-- Name: user_monitor_commands; Type: TABLE; Schema: public; Owner: postgres
--
CREATE TABLE user_monitor_commands (
    id_task integer NOT NULL,
    command_type integer NOT NULL,
    registration_time timestamp with time zone DEFAULT now() NOT NULL,
    id integer NOT NULL,
    is_processed boolean DEFAULT false NOT NULL
);
ALTER TABLE user_monitor_commands OWNER TO postgres;
--
-- Name: TABLE user_monitor_commands; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TABLE user_monitor_commands IS 'Table for registering user commands when working with the Monitor application.';
--
-- Name: COLUMN user_monitor_commands.id_task; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN user_monitor_commands.id_task IS 'Task ID';
--
-- Name: COLUMN user_monitor_commands.command_type; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN user_monitor_commands.command_type IS 'User command type see command_type_description table';
--
-- Name: COLUMN user_monitor_commands.registration_time; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN user_monitor_commands.registration_time IS 'Command registration time for debugging';
--
-- Name: COLUMN user_monitor_commands.id; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN user_monitor_commands.id IS 'Row unique ID';
--
-- Name: COLUMN user_monitor_commands.is_processed; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON COLUMN user_monitor_commands.is_processed IS 'An indication that the command has been processed.';
--
-- Name: user_monitor_commands_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--
CREATE SEQUENCE user_monitor_commands_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER TABLE user_monitor_commands_id_seq OWNER TO postgres;
--
-- Name: user_monitor_commands_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--
ALTER SEQUENCE user_monitor_commands_id_seq OWNED BY user_monitor_commands.id;
--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY agent_performance_counters ALTER COLUMN id SET DEFAULT nextval('agent_performance_counters_id_seq'::regclass);
--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY log ALTER COLUMN id SET DEFAULT nextval('log_id_seq'::regclass);
--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY resource_identificators ALTER COLUMN id SET DEFAULT nextval('resource_identificators_id_seq'::regclass);
--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY tasks ALTER COLUMN id SET DEFAULT nextval('tasks_id_seq'::regclass);
--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY user_monitor_commands ALTER COLUMN id SET DEFAULT nextval('user_monitor_commands_id_seq'::regclass);
--
-- Data for Name: agent_perf_data; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY agent_perf_data (hostname, last_pos, last_update, processor_time, memory_free, disk_time, disk_free_mb, network_bytes_sent, network_bytes_received) FROM stdin;
\.
--
-- Data for Name: agent_perf_setup; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY agent_perf_setup (frequency, history_len) FROM stdin;
1	604800
\.
--
-- Data for Name: agent_performance_counters; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY agent_performance_counters (agent_id, counter_name, counter_value, counter_path, measurement_time, id, group_id) FROM stdin;
\.
--
-- Name: agent_performance_counters_group_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('agent_performance_counters_group_id_seq', 46, true);
--
-- Name: agent_performance_counters_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('agent_performance_counters_id_seq', 150, true);
--
-- Data for Name: agents; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY agents (id, hostname, cpu_count, ram_size, last_update_time, fake_cpu_count, mt_coeff, is_paused, ip_address, version, is_alive, max_cpu_count, gpu_memory, avail_gpu_devices) FROM stdin;
\.
--
-- Name: agents_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('agents_id_seq', 182, true);
--
-- Data for Name: command_type_description; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY command_type_description (command_name, id_command) FROM stdin;
none	0
pause	1
resume	2
abort	3
start	4
delete	5
abort_delete	6
\.
--
-- Data for Name: failed_tasks; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY failed_tasks (id_task, id_agent) FROM stdin;
\.
--
-- Data for Name: log; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY log (id, message, id_session, msg_level, module_name, log_time, msg_type) FROM stdin;
\.
--
-- Name: log_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('log_id_seq', 3739548, true);
--
-- Data for Name: processing; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY processing (id, id_session, progress, status, update_time) FROM stdin;
\.
--
-- Name: processing_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('processing_id_seq', 7096, true);
--
-- Data for Name: processing_parameters; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY processing_parameters (manual_mode, db_version, db_version_sha) FROM stdin;
f	0	3509f09b34a939ce22f3738b17d50c9dc7ddf18e03de2dd755d268b936905edc
\.
--
-- Data for Name: resource_identificators; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY resource_identificators (res_name, id, status) FROM stdin;
\.
--
-- Name: resource_identificators_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('resource_identificators_id_seq', 10, true);
--
-- Data for Name: sessions; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY sessions (id_session, id_task, id_agent, start_time, elapsed_time, finish_time, has_warnings) FROM stdin;
\.
--
-- Name: sessions_id_session_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('sessions_id_session_seq', 7098, true);
--
-- Data for Name: task_dependencies; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY task_dependencies (id, id_group, id_dependency, id_task) FROM stdin;
\.
--
-- Name: task_dependencies_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('task_dependencies_id_seq', 7414, true);
--
-- Data for Name: task_status_description; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY task_status_description (id, status_name) FROM stdin;
0	none
1	started
2	paused
3	finished
4	failed
5	unknown
6	semifailed
7	restarted
8	stopped
9	resumed
\.
--
-- Data for Name: tasks; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY tasks (id, name, block_description, ram_per_cpu_needed, parameters, registration_time, hostname, hostpid, command, command_args, multi_threading, priority, manual_start) FROM stdin;
\.
--
-- Name: tasks_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('tasks_id_seq', 7415, true);
--
-- Data for Name: user_monitor_commands; Type: TABLE DATA; Schema: public; Owner: postgres
--
COPY user_monitor_commands (id_task, command_type, registration_time, id, is_processed) FROM stdin;
\.
--
-- Name: user_monitor_commands_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--
SELECT pg_catalog.setval('user_monitor_commands_id_seq', 8707, true);
--
-- Name: agent_perf_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY agent_perf_data
    ADD CONSTRAINT agent_perf_pkey PRIMARY KEY (hostname);
--
-- Name: failed_tasks_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY failed_tasks
    ADD CONSTRAINT failed_tasks_pkey PRIMARY KEY (id_task);
--
-- Name: pkey_agents; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY agents
    ADD CONSTRAINT pkey_agents PRIMARY KEY (id);
--
-- Name: pkey_apc_id; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY agent_performance_counters
    ADD CONSTRAINT pkey_apc_id PRIMARY KEY (id);
--
-- Name: pkey_command_type_description; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY command_type_description
    ADD CONSTRAINT pkey_command_type_description PRIMARY KEY (command_name);
--
-- Name: pkey_log; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY log
    ADD CONSTRAINT pkey_log PRIMARY KEY (id);
--
-- Name: pkey_processing; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY processing
    ADD CONSTRAINT pkey_processing PRIMARY KEY (id);
--
-- Name: pkey_resource_identificators; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY resource_identificators
    ADD CONSTRAINT pkey_resource_identificators PRIMARY KEY (res_name);
--
-- Name: pkey_sessions; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY sessions
    ADD CONSTRAINT pkey_sessions PRIMARY KEY (id_session);
--
-- Name: pkey_task_dependency; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY task_dependencies
    ADD CONSTRAINT pkey_task_dependency PRIMARY KEY (id);
--
-- Name: pkey_task_status_description; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY task_status_description
    ADD CONSTRAINT pkey_task_status_description PRIMARY KEY (status_name);
--
-- Name: pkey_tasks; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY tasks
    ADD CONSTRAINT pkey_tasks PRIMARY KEY (id);
--
-- Name: pkey_user_monitor_commands; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY user_monitor_commands
    ADD CONSTRAINT pkey_user_monitor_commands PRIMARY KEY (id);
--
-- Name: ukey_command_type_description_id_command; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY command_type_description
    ADD CONSTRAINT ukey_command_type_description_id_command UNIQUE (id_command);
--
-- Name: ukey_status_id; Type: CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY task_status_description
    ADD CONSTRAINT ukey_status_id UNIQUE (id);
--
-- Name: fki_log_id_session_fkey; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX fki_log_id_session_fkey ON log USING btree (id_session);
--
-- Name: fki_log_time; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX fki_log_time ON log USING btree (log_time);
--
-- Name: fki_pocessing_id_session_index; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX fki_pocessing_id_session_index ON public.processing USING btree (id_session);
--
-- Name: fki_pocessing_status_index; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX fki_pocessing_status_index ON public.processing USING btree (status);
--
-- Name: fki_sessions_id_task_fkey; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX fki_sessions_id_task_fkey ON sessions USING btree (id_task);
--
-- Name: fki_task_dependencies_fkey; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX fki_task_dependencies_fkey ON public.task_dependencies USING btree (id_task, id_dependency, id_group);
--
-- Name: fki_tasks_priority_desc_index; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX fki_tasks_priority_desc_index ON public.tasks USING btree (priority DESC);
--
-- Name: index_agpm_group_id; Type: INDEX; Schema: public; Owner: postgres
--
CREATE UNIQUE INDEX index_agpm_group_id ON avg_group_performance_measurements USING btree (group_id);
--
-- Name: index_agpm_id_agent; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX index_agpm_id_agent ON avg_group_performance_measurements USING btree (id_agent);
--
-- Name: index_agpm_meas_time; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX index_agpm_meas_time ON avg_group_performance_measurements USING btree (avg_measurement_time);
--
-- Name: index_apc_counter_name; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX index_apc_counter_name ON agent_performance_counters USING btree (counter_name);
--
-- Name: index_apc_group_id; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX index_apc_group_id ON agent_performance_counters USING btree (group_id);
--
-- Name: index_apc_measurement_time; Type: INDEX; Schema: public; Owner: postgres
--
CREATE INDEX index_apc_measurement_time ON agent_performance_counters USING btree (measurement_time);
--
-- Name: agents after_gpu_update; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER after_gpu_update AFTER UPDATE OF gpu_memory ON public.agents FOR EACH ROW EXECUTE PROCEDURE send_notify_on_agents_gpu_change();
--
-- Name: after_is_paused_update; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER after_is_paused_update AFTER UPDATE OF is_paused ON agents FOR EACH ROW EXECUTE PROCEDURE send_notify_on_agent_removed_from_pause();
--
-- Name: agents after_max_tasks_update; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER after_max_tasks_update AFTER UPDATE OF fake_cpu_count, mt_coeff ON public.agents FOR EACH ROW EXECUTE PROCEDURE public.send_notify_on_max_tasks_increased();
--
-- Name: on_agent_perf_setup_changed; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER on_agent_perf_setup_changed AFTER INSERT OR DELETE OR UPDATE ON agent_perf_setup FOR EACH ROW EXECUTE PROCEDURE agent_perf_setup_changed();
--
-- Name: on_agents_perf_data_changed; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER on_agents_perf_data_changed AFTER INSERT OR UPDATE ON agent_perf_data FOR EACH ROW EXECUTE PROCEDURE send_notify_on_agents_perf_changed();
--
-- Name: on_manual_start_changed; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER on_manual_start_changed AFTER UPDATE OF manual_start ON tasks FOR EACH ROW EXECUTE PROCEDURE send_notify_on_manual_start_changed();
--
-- Name: on_processing_mode_changed; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER on_processing_mode_changed AFTER UPDATE OF manual_mode ON processing_parameters FOR EACH ROW EXECUTE PROCEDURE send_notify_on_processing_mode_change();
--
-- Name: on_status_update; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER on_status_update AFTER UPDATE OF status ON processing FOR EACH ROW EXECUTE PROCEDURE run_task_planning();
--
-- Name: log trig_after_row_insert; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_after_row_insert AFTER INSERT ON public.log FOR EACH ROW EXECUTE PROCEDURE public.after_log_insert();
--
-- Name: trig_apc_refresh_materialized_view; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_apc_refresh_materialized_view AFTER INSERT OR DELETE OR UPDATE ON agent_performance_counters FOR EACH STATEMENT EXECUTE PROCEDURE apc_refresh_materialized_view();
--
-- Name: trig_before_agents_insert; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_before_agents_insert BEFORE INSERT ON agents FOR EACH ROW EXECUTE PROCEDURE before_agents_insert();
--
-- Name: trig_before_processing_update; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_before_processing_update BEFORE UPDATE ON processing FOR EACH ROW EXECUTE PROCEDURE before_processing_update();
--
-- Name: trig_before_status_update; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_before_status_update BEFORE UPDATE OF status ON processing FOR EACH ROW EXECUTE PROCEDURE before_status_update();
--
-- Name: trig_on_new_user_command; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_on_new_user_command AFTER INSERT ON user_monitor_commands FOR EACH ROW EXECUTE PROCEDURE execute_user_command();
--
-- Name: TRIGGER trig_on_new_user_command ON user_monitor_commands; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TRIGGER trig_on_new_user_command ON user_monitor_commands IS 'Execute user command.';
--
-- Name: trig_refresh_log_performance; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_refresh_log_performance AFTER INSERT OR DELETE OR UPDATE ON log FOR EACH STATEMENT EXECUTE PROCEDURE log_performance_refresh_materialized_view();
ALTER TABLE log DISABLE TRIGGER trig_refresh_log_performance;
--
-- Name: trig_send_notify_on_agents_alive_change; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_send_notify_on_agents_alive_change AFTER INSERT OR DELETE OR UPDATE ON agents_alive FOR EACH STATEMENT EXECUTE PROCEDURE send_notify_on_agents_change();
--
-- Name: trig_send_notify_on_agents_change; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_send_notify_on_agents_change AFTER INSERT OR DELETE OR UPDATE ON agents FOR EACH ROW EXECUTE PROCEDURE send_notify_on_agents_change();
--
-- Name: TRIGGER trig_send_notify_on_agents_change ON agents; Type: COMMENT; Schema: public; Owner: postgres
--
COMMENT ON TRIGGER trig_send_notify_on_agents_change ON agents IS 'Trigger that fires on any change to a table. Sends a notification to the client, according to which it updates its view model.';
--
-- Name: trig_send_notify_on_processing_changed; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_send_notify_on_processing_changed AFTER INSERT OR DELETE OR UPDATE ON processing FOR EACH ROW EXECUTE PROCEDURE send_notify_on_processing_change();
--
-- Name: trig_send_notify_on_tasks_changed; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER trig_send_notify_on_tasks_changed AFTER INSERT OR DELETE OR UPDATE ON tasks FOR EACH ROW EXECUTE PROCEDURE send_notify_on_tasks_change();
--
-- Name: upd_sessions; Type: TRIGGER; Schema: public; Owner: postgres
--
CREATE TRIGGER upd_sessions AFTER UPDATE ON sessions FOR EACH ROW EXECUTE PROCEDURE upd_session();
--
-- Name: fkey_agent_performance_counters; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY agent_performance_counters
    ADD CONSTRAINT fkey_agent_performance_counters FOREIGN KEY (agent_id) REFERENCES agents(id) ON UPDATE CASCADE ON DELETE CASCADE;
--
-- Name: fkey_log_id_session; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY log
    ADD CONSTRAINT fkey_log_id_session FOREIGN KEY (id_session) REFERENCES sessions(id_session) ON UPDATE CASCADE ON DELETE CASCADE;
--
-- Name: fkey_processing_id_session; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY processing
    ADD CONSTRAINT fkey_processing_id_session FOREIGN KEY (id_session) REFERENCES sessions(id_session) ON UPDATE CASCADE ON DELETE CASCADE;
--
-- Name: fkey_sessions_id_agent; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY sessions
    ADD CONSTRAINT fkey_sessions_id_agent FOREIGN KEY (id_agent) REFERENCES agents(id) ON UPDATE CASCADE ON DELETE CASCADE;
--
-- Name: fkey_sessions_id_task; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--
ALTER TABLE ONLY sessions
    ADD CONSTRAINT fkey_sessions_id_task FOREIGN KEY (id_task) REFERENCES tasks(id) ON UPDATE CASCADE ON DELETE CASCADE;
--
-- Name: avg_group_performance_measurements; Type: MATERIALIZED VIEW DATA; Schema: public; Owner: postgres
--
REFRESH MATERIALIZED VIEW avg_group_performance_measurements;
--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--
REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;
--
-- PostgreSQL database dump complete
--
